package jimenezcarlos.android.uoc.edu.camina;

import android.content.Context;
import android.net.Uri;
import android.os.Bundle;

import androidx.fragment.app.Fragment;

import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.ProgressBar;
import android.widget.ScrollView;
import android.widget.TextView;
import android.widget.Toast;

import com.google.firebase.auth.FirebaseAuth;
import com.google.firebase.auth.FirebaseUser;
import com.google.firebase.firestore.DocumentReference;
import com.google.firebase.firestore.DocumentSnapshot;
import com.google.firebase.firestore.FirebaseFirestore;

import java.util.ArrayList;

import jimenezcarlos.android.uoc.edu.rest.model.CaminaUser;
import jimenezcarlos.android.uoc.edu.util.AvailableBadgeCollection;
import jimenezcarlos.android.uoc.edu.util.Badge;


public class AchievementsBadgesFragment extends Fragment {

    // To save references to layout items
    private ImageView[] mBadges = new ImageView[15];
    private TextView[] mBadgesNames = new TextView[15];
    private TextView[] mBadgesRequirements = new TextView[15];
    private ProgressBar mProgressBar;
    private ScrollView mContent;

    private CaminaUser mCaminaUser;

    private OnFragmentInteractionListener mListener;

    public AchievementsBadgesFragment() {
        // Required empty public constructor
    }



    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        // Inflate the layout for this fragment
        View rootView =  inflater.inflate(R.layout.fragment_achievements_badges, container, false);
        mContent = rootView.findViewById(R.id.badge_content);
        mProgressBar = rootView.findViewById(R.id.badge_progress);

        //Get the references to views in layout
        for (int i=0; i<15; i++) {
            String resource = "imageView_badge_" + (i + 1);
            int id = getResources().getIdentifier(resource, "id", getActivity().getPackageName());
            mBadges[i] = rootView.findViewById(id);
            resource = "badge_textView_"+ (i + 1);
            id = getResources().getIdentifier(resource, "id", getActivity().getPackageName());
            mBadgesNames[i]= rootView.findViewById(id);
            resource = "badge_textView_req_"+ (i + 1);
            id = getResources().getIdentifier(resource, "id", getActivity().getPackageName());
            mBadgesRequirements[i]= rootView.findViewById(id);
        }

        readGlobalStatistics();

        return rootView;

    }

    private void readGlobalStatistics() {
        //Hide content and show the progress bar
        showLoading();
        // Get an instance of FireStore
        FirebaseFirestore fireStoreDB = FirebaseFirestore.getInstance();
        //Get user
        FirebaseUser user = FirebaseAuth.getInstance().getCurrentUser();
        //TODO: if user==null, tell the user he must be logged-in and send him to sign-in
        //Get a reference to the user's document
        DocumentReference userDocumentReference = fireStoreDB.collection("users").document(user.getUid());
        //Execute query to get user's global stats
        userDocumentReference
                .get()
                .addOnCompleteListener(task -> {
                    if (task.isSuccessful()) {
                        DocumentSnapshot document = task.getResult();
                        if (document.exists()) {
                            //Get the dailyStatistics from server
                            mCaminaUser = task.getResult().toObject(CaminaUser.class);
                            showBadges();

                        } else {

                            mProgressBar.setVisibility(View.INVISIBLE);
                            Toast.makeText(this.getActivity(),
                                    R.string.show_statistics_error,
                                    Toast.LENGTH_LONG).show();

                        }

                    } else {

                        mProgressBar.setVisibility(View.INVISIBLE);
                        Toast.makeText(this.getActivity(),
                                R.string.show_statistics_error,
                                Toast.LENGTH_LONG).show();

                    }



                });

    }

    private void showBadges() {

        ArrayList<Badge> usersBadgesCollection;
        AvailableBadgeCollection availableBadges = new AvailableBadgeCollection();
        usersBadgesCollection = availableBadges.getBadges(mCaminaUser);
        for (int i=0; i<15; i++) {

            mBadges[i].setImageResource(usersBadgesCollection.get(i).getBadgeDrawableResourceId());
            mBadgesNames[i].setText(usersBadgesCollection.get(i).getBadgeName());
            if (!usersBadgesCollection.get(i).getBadgeName().equals ("???\n???"))
                mBadgesRequirements[i].setBackground(getResources().getDrawable(R.drawable.rounded_obtained_badge_textview));
            mBadgesRequirements[i].setText(usersBadgesCollection.get(i).getRequirementLabel());

        }

        //Show content and hide progressbar
        mProgressBar.setVisibility(View.GONE);
        mContent.setVisibility(View.VISIBLE);
    }


    @Override
    public void onAttach(Context context) {
        super.onAttach(context);
        if (context instanceof OnFragmentInteractionListener) {
            mListener = (OnFragmentInteractionListener) context;
        } else {
            throw new RuntimeException(context.toString()
                    + " must implement OnFragmentInteractionListener");
        }
    }

    @Override
    public void onDetach() {
        super.onDetach();
        mListener = null;
    }

    /**
     * This interface must be implemented by activities that contain this
     * fragment to allow an interaction in this fragment to be communicated
     * to the activity and potentially other fragments contained in that
     * activity.
     */
    public interface OnFragmentInteractionListener {
        // TODO: Update argument type and name
        void onFragmentInteraction(Uri uri);
    }

    private void showLoading() {
        //Show progressbar and hide content
        mProgressBar.setVisibility(View.VISIBLE);
        mContent.setVisibility(View.GONE);
    }


}
