package jimenezcarlos.android.uoc.edu.camina;

import android.content.Context;
import android.net.Uri;
import android.os.Bundle;

import androidx.constraintlayout.widget.ConstraintLayout;
import androidx.fragment.app.Fragment;

import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ProgressBar;
import android.widget.TextView;
import android.widget.Toast;

import com.google.firebase.auth.FirebaseAuth;
import com.google.firebase.auth.FirebaseUser;
import com.google.firebase.firestore.CollectionReference;
import com.google.firebase.firestore.FirebaseFirestore;
import com.google.firebase.firestore.Query;
import jimenezcarlos.android.uoc.edu.rest.model.DailyStatistics;
import jimenezcarlos.android.uoc.edu.util.Utitilies;


public class AchievementsRecordsFragment extends Fragment {
    // References to views in the layout
    private TextView mSkippedStations;
    private TextView mCalories;
    private TextView mDistance;
    private TextView mTime;
    private TextView mRecordStopsDate;
    private TextView mRecordCaloriesDate;
    private TextView mRecordDistanceDate;
    private TextView mRecordTimeDate;
    private ProgressBar mProgressBar;
    private ConstraintLayout mContent;
    private TextView mNoRecords;
    private int recordsFetched = 0;


    private OnFragmentInteractionListener mListener;

    public AchievementsRecordsFragment() {
        // Required empty public constructor
    }


    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        // Inflate the layout for this fragment
        View rootView =  inflater.inflate(R.layout.fragment_achievements_records, container, false);

        // Get references to layout elements
        mSkippedStations = rootView.findViewById(R.id.ac_textView_metroStops);
        mCalories = rootView.findViewById(R.id.ac_textView_calories);
        mDistance = rootView.findViewById(R.id.ac_textView_distance);
        mTime = rootView.findViewById(R.id.ac_textView_time);

        mRecordStopsDate = rootView.findViewById(R.id.record_stops_date);
        mRecordCaloriesDate = rootView.findViewById(R.id.record_calories_date);
        mRecordDistanceDate = rootView.findViewById(R.id.record_distance_date);
        mRecordTimeDate = rootView.findViewById(R.id.record_time_date);

        mProgressBar = rootView.findViewById(R.id.ac_progress);
        mContent = rootView.findViewById(R.id.ac_content);
        mNoRecords = rootView.findViewById(R.id.ac_info_no_records);
        readRecords();
        return rootView;
    }

    private void readRecords() {


        //Hide content and show the progress bar
        showLoading();
        // Get an instance of FireStore
        FirebaseFirestore fireStoreDB = FirebaseFirestore.getInstance();
        //Get user
        FirebaseUser user = FirebaseAuth.getInstance().getCurrentUser();
        //TODO: if user==null, tell the user he must be logged-in and send him to sign-in
        //Get a reference to the user's dailyRegistries collection in FireStore
        CollectionReference dailyRegistriesColReference = fireStoreDB.collection("users").document(user.getUid()).collection("dailyRegistries");

        //Execute a query to get the DailyStatistic object with the highest skippedStations value
        dailyRegistriesColReference.orderBy("skippedStations", Query.Direction.DESCENDING)
                .orderBy("distance", Query.Direction.DESCENDING)
                .get()
                .addOnCompleteListener(task -> {
                    if (task.isSuccessful()) {
                        if (task.getResult().size()>0) { //There are dailyRegistries
                            DailyStatistics recordSkippedStations= task.getResult().getDocuments().get(0).toObject(DailyStatistics.class);
                            mSkippedStations.setText(getString(R.string.st_metro_stops,recordSkippedStations.getSkippedStations()));
                            mRecordStopsDate.setText(task.getResult().getDocuments().get(0).getId().replace("_", "/"));
                            recordsFetched++;
                            if (recordsFetched==4) showRecords();
                        } else {
                            showNoRecordsMessage();
                        }
                    }
                    else {
                        mProgressBar.setVisibility(View.INVISIBLE);
                        Log.e("****************RECORDS", "Error getting documents: ", task.getException());
                        Toast.makeText(this.getActivity(),
                                R.string.show_records_error + task.getException().getMessage(),
                                Toast.LENGTH_LONG).show();
                    }
                });


        //Execute a query to get the DailyStatistic object with the highest calories value
        dailyRegistriesColReference.orderBy("calories", Query.Direction.DESCENDING)
                .orderBy("distance", Query.Direction.DESCENDING)
                .get()
                .addOnCompleteListener(task -> {
                    if (task.isSuccessful()) {
                        if (task.getResult().size()>0) { //There are dailyRegistries
                            DailyStatistics recordCalories= task.getResult().getDocuments().get(0).toObject(DailyStatistics.class);
                            mCalories.setText(getString(R.string.st_calories,recordCalories.getCalories()));
                            mRecordCaloriesDate.setText(task.getResult().getDocuments().get(0).getId().replace("_", "/"));
                            recordsFetched++;
                            if (recordsFetched==4) showRecords();

                        } else {
                            showNoRecordsMessage();
                        }

                    }
                    else {
                        mProgressBar.setVisibility(View.INVISIBLE);
                        Log.e("****************RECORDS", "Error getting documents: ", task.getException());
                        Toast.makeText(this.getActivity(),
                                R.string.show_records_error + task.getException().getMessage(),
                                Toast.LENGTH_LONG).show();
                    }
                });

        //Execute a query to get the DailyStatistic object with the highest distance value
        //In this case, we will get the older registry with the highest distance value
        dailyRegistriesColReference.orderBy("distance", Query.Direction.DESCENDING)
                .orderBy("year", Query.Direction.ASCENDING)
                .orderBy("month", Query.Direction.ASCENDING)
                .orderBy("day", Query.Direction.ASCENDING)
                .get()
                .addOnCompleteListener(task -> {
                    if (task.isSuccessful()) {
                        if (task.getResult().size()>0) { //There are dailyRegistries
                            DailyStatistics recordDistance= task.getResult().getDocuments().get(0).toObject(DailyStatistics.class);
                            mDistance.setText(Utitilies.FormatDistanceDoubleToString((double)recordDistance.getDistance()));
                            mRecordDistanceDate.setText(task.getResult().getDocuments().get(0).getId().replace("_", "/"));
                            recordsFetched++;
                            if (recordsFetched==4) showRecords();

                        } else {
                            showNoRecordsMessage();
                        }

                    }
                    else {
                        Log.e("****************RECORDS", "Error getting documents: ", task.getException());
                        mProgressBar.setVisibility(View.INVISIBLE);
                        Toast.makeText(this.getActivity(),
                                R.string.show_records_error + task.getException().getMessage(),
                                Toast.LENGTH_LONG).show();
                    }
                });

        //Execute a query to get the DailyStatistic object with the highest time value
        dailyRegistriesColReference.orderBy("time", Query.Direction.DESCENDING)
                .orderBy("distance", Query.Direction.DESCENDING)
                .get()
                .addOnCompleteListener(task -> {
                    if (task.isSuccessful()) {
                        if (task.getResult().size()>0) { //There are dailyRegistries
                            DailyStatistics recordTime= task.getResult().getDocuments().get(0).toObject(DailyStatistics.class);
                            mTime.setText(Utitilies.FormatTimeIntegerToString(recordTime.getTime()));
                            mRecordTimeDate.setText(task.getResult().getDocuments().get(0).getId().replace("_", "/"));
                            recordsFetched++;
                            if (recordsFetched==4) showRecords();

                        } else {
                            showNoRecordsMessage();
                        }

                    }
                    else {
                        Log.e("****************RECORDS", "Error getting documents: ", task.getException());
                        mProgressBar.setVisibility(View.INVISIBLE);
                        Toast.makeText(this.getActivity(),
                                R.string.show_records_error + task.getException().getMessage(),
                                Toast.LENGTH_LONG).show();
                    }
                });

    }


    @Override
    public void onAttach(Context context) {
        super.onAttach(context);
        if (context instanceof OnFragmentInteractionListener) {
            mListener = (OnFragmentInteractionListener) context;
        } else {
            throw new RuntimeException(context.toString()
                    + " must implement OnFragmentInteractionListener");
        }
    }

    @Override
    public void onDetach() {
        super.onDetach();
        mListener = null;
    }

    /**
     * This interface must be implemented by activities that contain this
     * fragment to allow an interaction in this fragment to be communicated
     * to the activity and potentially other fragments contained in that
     * activity.
     */
    public interface OnFragmentInteractionListener {
        // TODO: Update argument type and name
        void onFragmentInteraction(Uri uri);
    }

    private void showLoading() {
        //Show progressbar and hide content
        mProgressBar.setVisibility(View.VISIBLE);
        mContent.setVisibility(View.GONE);
    }

    private void showRecords() {

        //Show content and hide progressbar
        mProgressBar.setVisibility(View.GONE);
        mContent.setVisibility(View.VISIBLE);
    }

    private void showNoRecordsMessage() {
        //Hide progressbar
        mProgressBar.setVisibility(View.GONE);
        mContent.setVisibility(View.GONE);
        mNoRecords.setVisibility(View.VISIBLE);

    }
}
