package jimenezcarlos.android.uoc.edu.camina;

import android.app.AlertDialog;
import android.content.Context;
import android.content.Intent;
import android.graphics.Color;
import android.graphics.PorterDuff;
import android.os.Bundle;
import android.util.TypedValue;
import android.view.View;
import android.widget.Button;
import android.widget.ImageView;
import android.widget.ProgressBar;
import android.widget.TextView;
import android.widget.Toast;

import androidx.annotation.NonNull;
import androidx.appcompat.app.AppCompatActivity;
import androidx.appcompat.widget.Toolbar;
import androidx.constraintlayout.widget.ConstraintLayout;
import androidx.constraintlayout.widget.ConstraintSet;
import androidx.constraintlayout.widget.Guideline;

import com.google.android.gms.tasks.OnCompleteListener;
import com.google.android.gms.tasks.OnFailureListener;
import com.google.android.gms.tasks.OnSuccessListener;
import com.google.android.gms.tasks.Task;
import com.google.firebase.auth.FirebaseAuth;
import com.google.firebase.auth.FirebaseUser;
import com.google.firebase.firestore.CollectionReference;
import com.google.firebase.firestore.DocumentReference;
import com.google.firebase.firestore.DocumentSnapshot;
import com.google.firebase.firestore.FieldValue;
import com.google.firebase.firestore.FirebaseFirestore;

import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.Locale;
import java.util.Map;

import jimenezcarlos.android.uoc.edu.menu.LateralMenu;
import jimenezcarlos.android.uoc.edu.rest.model.CaminaUser;
import jimenezcarlos.android.uoc.edu.rest.model.DailyStatistics;
import jimenezcarlos.android.uoc.edu.rest.model.MonthlyStatistics;
import jimenezcarlos.android.uoc.edu.rest.model.Path;
import jimenezcarlos.android.uoc.edu.rest.model.Registry;
import jimenezcarlos.android.uoc.edu.util.LineToTransitIconDrawable;

import static jimenezcarlos.android.uoc.edu.util.Utitilies.FormatDistanceDoubleToString;
import static jimenezcarlos.android.uoc.edu.util.Utitilies.FormatTimeIntegerToString;

public class CertifyItineraryActivity extends AppCompatActivity implements View.OnClickListener {

    // To save references to layout items
    private ImageView[] mTransitIcon = new ImageView[4];
    private ImageView[] mStationPoints = new ImageView[8];
    private ImageView[] mMetroLines = new ImageView[4];
    private TextView[] mStationNames = new TextView[5];
    private TextView[] mStationsInLine = new TextView[4];
    private Guideline[] mHorizontalGuidelines = new Guideline[6];
    private TextView mInfoItinerary;
    private TextView mStartIndications;
    private TextView mEndIndications;
    private TextView mOrigin;
    private TextView mDestination;

    private ProgressBar mProgressBar;
    private View content;
    private TextView mNoItinerary;

    private Button mCertifyButton;
    private Button mDeleteButton;

    private CaminaUser caminaUser;

    //FirestoreDB instance
    private FirebaseFirestore firestoreDB;
    FirebaseUser user;


    // Auxiliary class to get the name of drawable resources of transit icon lines
    private LineToTransitIconDrawable iconLines;


    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        // Set the layout for this activity
        setContentView(R.layout.activity_certify_itinerary);

        // Get reference to the support action bar in the layout and set it
        Toolbar toolbar = findViewById(R.id.ci_toolbar);
        setSupportActionBar(toolbar);

        //Include the Lateral Menu
        LateralMenu.getDrawer(this, savedInstanceState, toolbar, 1);

        //Instantiate a new object of class LineToTransitIconDrawable
        iconLines = new LineToTransitIconDrawable();

        // Get an instance of Firestore
        firestoreDB = FirebaseFirestore.getInstance();
        //Get user
        user = FirebaseAuth.getInstance().getCurrentUser();
        initiViews();
        readLastRoute();

    }

    private void initiViews() {

        mInfoItinerary = findViewById(R.id.ci_info_itinerary);
        mStartIndications = findViewById(R.id.ci_textView_walk_indications_1);
        mEndIndications = findViewById(R.id.ci_textView_walk_indications_2);
        mOrigin = findViewById(R.id.ci_textOrigin);
        mDestination = findViewById(R.id.ci_textDestination);
        mProgressBar = findViewById(R.id.certify_progress);
        content = findViewById(R.id.ci_Content);
        mCertifyButton = findViewById(R.id.ci_buttonCertify);
        mDeleteButton = findViewById(R.id.ci_buttonDelete);
        mNoItinerary = findViewById(R.id.ci_info_no_itinerary);

        //Get the reference to Metro station names
        for (int i = 0; i <= 4; i++) {
            String resource = "ci_metroStop_" + String.valueOf(i + 1);
            int id = getResources().getIdentifier(resource, "id", getBaseContext().getPackageName());
            mStationNames[i] = findViewById(id);

        }


        // Get references to start/end of metro routes
        for (int i = 0; i <= 7; i++) {
            String resource = "ci_icon_metro_stop_" + String.valueOf(i + 1);
            int id = getResources().getIdentifier(resource, "id", getBaseContext().getPackageName());
            mStationPoints[i] = findViewById(id);

        }
        // Get reference to transit lines icons, transit lines lines, and text with number of stops
        for (int i = 0; i <= 3; i++) {
            String resource = "ci_transtit_line_icon_" + String.valueOf(i + 1);
            int id = getResources().getIdentifier(resource, "id", getBaseContext().getPackageName());
            mTransitIcon[i] = findViewById(id);
            resource = "ci_transit_line_" + String.valueOf(i + 1);
            id = getResources().getIdentifier(resource, "id", getBaseContext().getPackageName());
            mMetroLines[i] = findViewById(id);
            resource = "ci_textView_stops_line_" + String.valueOf(i + 1);
            id = getResources().getIdentifier(resource, "id", getBaseContext().getPackageName());
            mStationsInLine[i] = findViewById(id);

        }

        // Get references to every guideline in the layout
        for (int i = 0; i <= 5; i++) {
            String resource = "ci_guideline_" + String.valueOf(i + 1);
            int id = getResources().getIdentifier(resource, "id", getBaseContext().getPackageName());
            mHorizontalGuidelines[i] = findViewById(id);

            // Set onClickListeners in the buttons

            mDeleteButton.setOnClickListener(this);
            mCertifyButton.setOnClickListener(this);

        }

    }

    void readLastRoute() {
        showLoading();
        //Get user
        user = FirebaseAuth.getInstance().getCurrentUser();
        //TODO: if user==null, tell the user he must be logged-in and send him to sign-in
        firestoreDB.collection("users")
                .document(user.getUid())
                .get()
                .addOnCompleteListener(new OnCompleteListener<DocumentSnapshot>() {
                    @Override
                    public void onComplete(@NonNull Task<DocumentSnapshot> task) {
                        if (task.isSuccessful()) {
                            DocumentSnapshot document = task.getResult();
                            if (document.exists()) { //If there was a document with that id, user.getUid()
                                caminaUser = task.getResult().toObject(CaminaUser.class);
                                if (caminaUser.getLastRoute() != null) { //If there was a lastRoute stored
                                    showItineraryContent(caminaUser.getLastRoute());
                                } else {
                                    showNoLastItineraryMessage();
                                }

                            }

                        } else {
                            Toast.makeText(CertifyItineraryActivity.this,
                                    getString(R.string.error_last_itinerary),
                                    Toast.LENGTH_SHORT).show();
                        }
                    }


                });

    }

    private void showLoading() {
        //Show progressbar and hide content
        mProgressBar.setVisibility(View.VISIBLE);
        content.setVisibility(View.GONE);
        mNoItinerary.setVisibility(View.GONE);
    }

    private void showNoLastItineraryMessage() {
        //Hide progressbar
        mProgressBar.setVisibility(View.GONE);
        content.setVisibility(View.GONE);
        mNoItinerary.setVisibility(View.VISIBLE);

    }

    private void showItineraryContent(Path lastRoute) {
        //Hide progressbar and set data
        mProgressBar.setVisibility(View.GONE);

        //Get date and give format to compose the appropriate string
        Date date = lastRoute.getDate();
        SimpleDateFormat formatter = new SimpleDateFormat("EEEE", Locale.getDefault());
        String weekDayString = formatter.format(date);
        formatter = new SimpleDateFormat("d", Locale.getDefault());
        String dayInMonthString = formatter.format(date);
        formatter = new SimpleDateFormat("MMMM", Locale.getDefault());
        String monthString = formatter.format(date);
        mInfoItinerary.setText(getString(R.string.intro_last_journey, weekDayString + " " + dayInMonthString + " de " + monthString));

        mOrigin.setText(lastRoute.getOriginAddress());
        mDestination.setText(lastRoute.getDestinationAddress());

        mStartIndications.setText(getString(R.string.text_walking_indications, FormatDistanceDoubleToString(lastRoute.getStartWalkingDistance()), FormatTimeIntegerToString(lastRoute.getStartWalkingTime())));
        mEndIndications.setText(getString(R.string.text_walking_indications, FormatDistanceDoubleToString(lastRoute.getEndWalkingDistance()), FormatTimeIntegerToString(lastRoute.getEndWalkingTime())));

        // Get the number of different metro lines, and adjust the layout (maximum is 4)

        int metroLines = lastRoute.getMetroLines().size();
        for (int i = 0; i < metroLines; i++) {
            String resource = iconLines.getResourceName(lastRoute.getMetroLines().get(i));
            int id = getResources().getIdentifier(resource, "drawable", getBaseContext().getPackageName());
            mTransitIcon[i].setImageDrawable(getDrawable(id));
            mTransitIcon[i].setVisibility(View.VISIBLE);
            // Get this Metro line color, and use it to tint the corresponding Metro line and points in Layout
            int metroLineColor = Color.parseColor("#" + lastRoute.getMetroLineColors().get(i));
            mMetroLines[i].setColorFilter(metroLineColor, PorterDuff.Mode.MULTIPLY);
            mMetroLines[i].setVisibility(View.VISIBLE);
            mStationPoints[i * 2].setColorFilter(metroLineColor, PorterDuff.Mode.MULTIPLY);
            mStationPoints[i * 2 + 1].setColorFilter(metroLineColor, PorterDuff.Mode.MULTIPLY);
            mStationPoints[i * 2].setVisibility(View.VISIBLE);
            mStationPoints[i * 2 + 1].setVisibility(View.VISIBLE);
        }

        if (lastRoute.getSkippedAll()) { //If user decide to skip all the stations in the first Metro Line

            mStationNames[0].setText(lastRoute.getStationNames().get(0).getStationNamesInLine().get(0));
            mStationsInLine[0].setText(getNumberOfStationsString(lastRoute.getStationNames().get(0).getStationNamesInLine().size()));
        }

        else {

            mStationNames[0].setText(lastRoute.getStationNames().get(0).getStationNamesInLine().get(lastRoute.getSkippedStations()));
            mStationsInLine[0].setText(getNumberOfStationsString(lastRoute.getStationNames().get(0).getStationNamesInLine().size() - lastRoute.getSkippedStations() - 1));

        }

        mStationNames[0].setVisibility(View.VISIBLE);
        mStationsInLine[0].setVisibility(View.VISIBLE);

        for (int i = 1; i < metroLines; i++) {
            mStationNames[i].setText(lastRoute.getStationNames().get(i - 1).getStationNamesInLine().get(lastRoute.getStationNames().get(i - 1).getStationNamesInLine().size() - 1));
            mStationNames[i].setVisibility(View.VISIBLE);
            mStationsInLine[i].setText(getNumberOfStationsString(lastRoute.getStationNames().get(i).getStationNamesInLine().size() - 1));
            mStationsInLine[i].setVisibility(View.VISIBLE);

        }

        mStationNames[metroLines].setText(lastRoute.getStationNames().get(metroLines - 1).getStationNamesInLine().get(lastRoute.getStationNames().get(metroLines - 1).getStationNamesInLine().size() - 1));
        mStationNames[metroLines].setVisibility(View.VISIBLE);


        // Rearrange Layout: move the 2 last horizontal guidelines to the appropriate position
        // According to the number of routes made in Metro
        ConstraintLayout.LayoutParams params = (ConstraintLayout.LayoutParams) mHorizontalGuidelines[4].getLayoutParams();
        params.guideBegin = (int) TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_DIP, 120 + metroLines * 100, getResources().getDisplayMetrics()); // Convert from dp's to pixels
        mHorizontalGuidelines[4].setLayoutParams(params);
        params = (ConstraintLayout.LayoutParams) mHorizontalGuidelines[5].getLayoutParams();
        params.guideBegin = (int) TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_DIP, 240 + metroLines * 100, getResources().getDisplayMetrics()); // Convert from dp's to pixels
        mHorizontalGuidelines[5].setLayoutParams(params);

        // Adjust vertical position of last name station, to be centered vertically to the last shown mStationPoint
        // 1. Get resource id of last shown station name textview
        String resource = "ci_metroStop_" + (metroLines + 1);
        int lastShownStationNameId = getResources().getIdentifier(resource, "id", getBaseContext().getPackageName());

        // 2. Get resource id of last shown station point imageview
        resource = "ci_icon_metro_stop_" + (metroLines * 2);
        int lastShownStationPointId = getResources().getIdentifier(resource, "id", getBaseContext().getPackageName());
        // 3. Set and apply new constraints
        ConstraintLayout constraintLayout = findViewById(R.id.ci_constraintLayout);
        ConstraintSet constraintSet = new ConstraintSet();
        constraintSet.clone(constraintLayout);
        constraintSet.connect(lastShownStationNameId, ConstraintSet.TOP, lastShownStationPointId, ConstraintSet.TOP, 0);
        constraintSet.connect(lastShownStationNameId, ConstraintSet.BOTTOM, lastShownStationPointId, ConstraintSet.BOTTOM, 0);
        constraintSet.applyTo(constraintLayout);

        content.setVisibility(View.VISIBLE);

    }


    @Override
    public void onClick(View v) {

        if (v.getId() == R.id.ci_buttonDelete) { //If buttonDelete was clicked

            // Show confirmation message to user
            AlertDialog.Builder alertDialog = new AlertDialog.Builder(CertifyItineraryActivity.this);
            alertDialog.setTitle(R.string.delete_last_itinerary_confirmation_title);
            alertDialog.setMessage(R.string.delete_last_itinerary_confirmation_message);
            alertDialog.setPositiveButton("OK",
                    (dialog, which) -> {
                        dialog.dismiss();
                        deleteLastItinerary(true);
                    });

            alertDialog.setNegativeButton("CANCELAR",
                    (dialog, id) -> dialog.dismiss());

            alertDialog.show();

        }

        if (v.getId()==R.id.ci_buttonCertify) { //If buttonCertify was clicked

            // Show confirmation message to user
            AlertDialog.Builder alertDialog = new AlertDialog.Builder(CertifyItineraryActivity.this);
            alertDialog.setTitle(R.string.certify_last_itinerary_confirmation_title);
            alertDialog.setMessage(R.string.certify_last_itinerary_confirmation_message);
            alertDialog.setPositiveButton("OK",
                    (dialog, which) -> {
                        dialog.dismiss();
                        certifyLastItinerary();
                    });

            alertDialog.setNegativeButton("CANCELAR",
                    (dialog, id) -> dialog.dismiss());

            alertDialog.show();


        }

    }

    private void certifyLastItinerary() {

        Calendar calendar = Calendar.getInstance();
        calendar.setTime(caminaUser.getLastRoute().getDate());
        int year = calendar.get(Calendar.YEAR);
        int month = calendar.get(Calendar.MONTH) + 1; //Calendar.MONTH is zero based, so it is giving 1 less than current month
        int dayOfMonth = calendar.get(Calendar.DAY_OF_MONTH);

        //Show the progress bar
        mProgressBar.setVisibility(View.VISIBLE);

        //Get a reference to the user's dailyRegistries collection in FireStore
        CollectionReference dailyRegistriesColReference = firestoreDB.collection("users").document(user.getUid()).collection("dailyRegistries");
        //Execute query to get DailyStatistics object for the day the itinerary we have to certify was made
        dailyRegistriesColReference
                .document(dayOfMonth + "_" + month + "_" + year)
                .get()
                .addOnCompleteListener(task -> {
                    if (task.isSuccessful()) {
                        DocumentSnapshot document = task.getResult();
                        if (!document.exists()) {
                        ////////////////////////////////////////////////////////////////////
                        //There is NOT a DailyStatistics object for this day, so create it//
                        ////////////////////////////////////////////////////////////////////

                        dailyRegistriesColReference
                                .document(dayOfMonth + "_" + month + "_" + year)
                                .set(new DailyStatistics(year, month, dayOfMonth, caminaUser.getLastRoute().getSkippedStations(), caminaUser.getLastRoute().getCalories().intValue(),
                                        caminaUser.getLastRoute().getWalkDistance().intValue(), caminaUser.getLastRoute().getWalkTime()))
                                .addOnSuccessListener(aVoid -> {
                                    //New document DailyStatistics successfully written
                                    updateMonthlyStatistics(month, year);

                                })
                                .addOnFailureListener(e -> {
                                    mProgressBar.setVisibility(View.INVISIBLE);
                                    Toast.makeText(CertifyItineraryActivity.this,
                                            R.string.certify_last_itinerary_error,
                                            Toast.LENGTH_LONG).show();
                                });
                        } else {
                            ///////////////////////////////////////////////////////////////////////////////////////////////////////////
                            //There is ALREADY a DailyStatistics object for this day, so update it by incrementing its properties    //
                            ///////////////////////////////////////////////////////////////////////////////////////////////////////////

                            DailyStatistics currentDS = task.getResult().toObject(DailyStatistics.class); //Get the dailyStatistics from server
                            DailyStatistics newDS = new DailyStatistics(year, month, dayOfMonth, currentDS.getSkippedStations()+caminaUser.getLastRoute().getSkippedStations(),
                                    currentDS.getCalories()+caminaUser.getLastRoute().getCalories().intValue(),currentDS.getDistance()+caminaUser.getLastRoute().getWalkDistance().intValue(),
                                    currentDS.getTime()+caminaUser.getLastRoute().getWalkTime());

                            dailyRegistriesColReference
                                    .document(dayOfMonth + "_" + month + "_" + year)
                                    .set(newDS)
                                    .addOnSuccessListener(aVoid -> {
                                        //Document DailyStatistics successfully updated
                                        updateMonthlyStatistics(month, year);

                                    })
                                    .addOnFailureListener(e -> {
                                        mProgressBar.setVisibility(View.INVISIBLE);
                                        Toast.makeText(CertifyItineraryActivity.this,
                                                R.string.certify_last_itinerary_error,
                                                Toast.LENGTH_SHORT).show();
                                    });


                        }

                    } else {

                        mProgressBar.setVisibility(View.INVISIBLE);
                        Toast.makeText(CertifyItineraryActivity.this,
                                R.string.certify_last_itinerary_error,
                                Toast.LENGTH_LONG).show();

                    }



                });
    }

    private void updateMonthlyStatistics(int month, int year) {

        //Get a reference to the user's monthlyRegistries collection in FireStore
        CollectionReference monthlyRegistriesColReference = firestoreDB.collection("users").document(user.getUid()).collection("monthlyRegistries");
        //Execute query to get MonthlyStatistics object for the month the itinerary we have to certify was made
        monthlyRegistriesColReference
                .document(month + "_" + year)
                .get()
                .addOnCompleteListener(task -> {
                    if (task.isSuccessful()) {
                        DocumentSnapshot document = task.getResult();
                        if (!document.exists()) {
                            ////////////////////////////////////////////////////////////////////////
                            //There is NOT a MonthlyStatistics object for this month, so create it//
                            ////////////////////////////////////////////////////////////////////////
                            monthlyRegistriesColReference
                                    .document( month + "_" + year)
                                    .set(new MonthlyStatistics(year, month, caminaUser.getLastRoute().getSkippedStations(), caminaUser.getLastRoute().getCalories().intValue(),
                                            caminaUser.getLastRoute().getWalkDistance().intValue(), caminaUser.getLastRoute().getWalkTime()))
                                    .addOnSuccessListener(aVoid -> {
                                        //New document MonthlyStatistics successfully written
                                        updateGlobalStatistics();


                                    })
                                    .addOnFailureListener(e -> {
                                        mProgressBar.setVisibility(View.INVISIBLE);
                                        Toast.makeText(CertifyItineraryActivity.this,
                                                R.string.certify_last_itinerary_error,
                                                Toast.LENGTH_LONG).show();
                                    });
                        } else {
                            /////////////////////////////////////////////////////////////////////////////////////////////////////////////
                            //     There is ALREADY a MonthlyStatistics object for the month the route we have to certify was made     //
                            //                          so update it by incrementing values of Registry                                //
                            /////////////////////////////////////////////////////////////////////////////////////////////////////////////

                            MonthlyStatistics currentMS = task.getResult().toObject(MonthlyStatistics.class); //Get the monthlyStatistics from server

                            MonthlyStatistics newMS = new MonthlyStatistics(year, month, currentMS.getSkippedStations()+caminaUser.getLastRoute().getSkippedStations(),
                                    currentMS.getCalories()+caminaUser.getLastRoute().getCalories().intValue(),currentMS.getDistance()+caminaUser.getLastRoute().getWalkDistance().intValue(),
                                    currentMS.getTime()+caminaUser.getLastRoute().getWalkTime());

                            monthlyRegistriesColReference
                                    .document(month + "_" + year)
                                    .set(newMS)
                                    .addOnSuccessListener(aVoid -> {
                                        //New document MonthlyStatistics successfully written
                                        updateGlobalStatistics();


                                    })
                                    .addOnFailureListener(e -> {
                                        mProgressBar.setVisibility(View.INVISIBLE);
                                        Toast.makeText(CertifyItineraryActivity.this,
                                                R.string.certify_last_itinerary_error,
                                                Toast.LENGTH_SHORT).show();
                                    });


                        }

                    } else {

                        mProgressBar.setVisibility(View.INVISIBLE);
                        Toast.makeText(CertifyItineraryActivity.this,
                                R.string.certify_last_itinerary_error,
                                Toast.LENGTH_LONG).show();

                    }



                });


    }

    private void updateGlobalStatistics() {
        //Every user has this 4 properties in his profile, initially set to 0.
        //So they will always exist, if user exists.
        //Thus, this method will always UPDATE, not create

        Map<String,Object> updates = new HashMap<>();
        updates.put("totalSkippedStations", caminaUser.getTotalSkippedStations()+ caminaUser.getLastRoute().getSkippedStations());
        updates.put("totalCalories", caminaUser.getTotalCalories()+ caminaUser.getLastRoute().getCalories());
        updates.put("totalDistance", caminaUser.getTotalDistance()+ caminaUser.getLastRoute().getWalkDistance());
        updates.put("totalTime", caminaUser.getTotalTime()+ caminaUser.getLastRoute().getWalkTime());

        // Get an instance of FireStore
        firestoreDB = FirebaseFirestore.getInstance();
        // Get user
        user = FirebaseAuth.getInstance().getCurrentUser();
        // Get reference to the user's document
        DocumentReference userDocumentReference = firestoreDB.collection("users").document(user.getUid());
        // Execute the update
        userDocumentReference.update(updates)
                .addOnSuccessListener(new OnSuccessListener<Void>() {
                    @Override
                    public void onSuccess(Void aVoid) {
                        //Hide the progress bar
                        mProgressBar.setVisibility(View.INVISIBLE);
                        //Since the last route has been already registered, delete it
                        deleteLastItinerary(false);
                        //Show success layout
                        showSuccessMessage();
                    }
                })
                .addOnFailureListener(new OnFailureListener() {
                    @Override
                    public void onFailure(@NonNull Exception e) {
                        mProgressBar.setVisibility(View.INVISIBLE);
                        Toast.makeText(CertifyItineraryActivity.this,
                                R.string.certify_last_itinerary_error,
                                Toast.LENGTH_SHORT).show();
                    }
                });




    }



    private void deleteLastItinerary(boolean showAlert) {
        //Show the progress bar
        mProgressBar.setVisibility(View.VISIBLE);
        //Get a reference to de user's document in Firestore
        DocumentReference docRef = firestoreDB.collection("users").document(user.getUid());
        //Compose a Map update to delete the field "lastRoute"
        Map<String, Object> updates = new HashMap<>();
        updates.put("lastRoute", FieldValue.delete());
        //And execute the update
        docRef.update(updates)
              .addOnSuccessListener(aVoid -> {
                  mProgressBar.setVisibility(View.INVISIBLE);
                  if (showAlert) {
                      AlertDialog alertDialog = new AlertDialog.Builder(CertifyItineraryActivity.this).create();
                      alertDialog.setTitle(R.string.delete_itinerary_success_title);
                      alertDialog.setMessage(getString(R.string.delete_itinerary_success_message));
                      alertDialog.setButton(AlertDialog.BUTTON_NEUTRAL, "OK",
                              (dialog, which) -> {
                                  dialog.dismiss();
                                  showNoLastItineraryMessage();
                              });
                      alertDialog.show();
                  }

              })

              .addOnFailureListener(e -> {
                  mProgressBar.setVisibility(View.INVISIBLE);
                  Toast.makeText(CertifyItineraryActivity.this,
                  R.string.delete_itinerary_error,
                  Toast.LENGTH_SHORT).show();
         });

    }


    public void showSuccessMessage () {

        setContentView(R.layout.itinerary_successfully_certified);
        Button buttonStatistics = findViewById(R.id.isc_button_showStatistics);
        buttonStatistics.setOnClickListener(v -> {
            // When buttonStatistics is clicked
            // a new intent to start StatisticsActivity is created
            final Context context = v.getContext();
            Intent intent = new Intent(context, StatisticsActivity.class);
            startActivity(intent);
            finish();
        });



    }


    // This auxiliary method manages singular/plural number of stations message
    public String getNumberOfStationsString (int nOfStations) {

        if (nOfStations==1)
            return getString(R.string.one_station);
        else return (getString(R.string.number_stations, nOfStations));

    }

}

