package jimenezcarlos.android.uoc.edu.camina;

import android.content.Context;
import android.net.Uri;
import android.os.Bundle;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.FrameLayout;
import android.widget.ImageView;
import android.widget.ProgressBar;
import android.widget.TextView;
import android.widget.Toast;

import androidx.fragment.app.Fragment;
import androidx.recyclerview.widget.RecyclerView;
import androidx.swiperefreshlayout.widget.SwipeRefreshLayout;

import com.google.firebase.firestore.CollectionReference;
import com.google.firebase.firestore.FirebaseFirestore;
import com.google.firebase.firestore.Query;
import com.google.firebase.firestore.QueryDocumentSnapshot;

import com.google.firebase.firestore.QuerySnapshot;
import com.squareup.picasso.Picasso;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.HashMap;
import java.util.List;

import jimenezcarlos.android.uoc.edu.rest.model.CaminaUser;
import jimenezcarlos.android.uoc.edu.rest.model.MonthlyStatistics;
import jimenezcarlos.android.uoc.edu.util.ClassificationPageAdapter;
import jimenezcarlos.android.uoc.edu.util.Utitilies;

import static java.lang.Boolean.FALSE;
import static java.lang.Boolean.TRUE;


public class ClassificationFragment extends Fragment {

    private static final String ARG_PERIOD = "period";
    private String mPeriod;

    // References to views in the layout
    private ProgressBar mProgressBar;
    private FrameLayout mContent;
    private RecyclerView mRecyclerView;
    private SwipeRefreshLayout mSwipeRefreshLayout;
    // This is the adapter to the RecyclerView
    private SimpleItemRecyclerViewAdapter mAdapter;

    // Auxiliary variable to flag whether a swipe to refresh the routes has been done
    private boolean swipePerformed = FALSE;

    // To identify a row in an odd position in the list
    private final static int ODD = 1;
    // To identify a row in an even position in the list
    private final static int EVEN = 2;


    private OnFragmentInteractionListener mListener;
    public ClassificationFragment() {
        // Required empty public constructor
    }

    /**
     * Use this factory method to create a new instance of
     * this fragment using the provided parameters.
     *
     * @param period Indicates whether the classification is for the current month or global
     * @return A new instance of fragment ClassificationFragment.
     */

    public static ClassificationFragment newInstance(String period) {
        ClassificationFragment fragment = new ClassificationFragment();
        Bundle args = new Bundle();
        args.putString(ARG_PERIOD, period);
        fragment.setArguments(args);
        return fragment;
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        if (getArguments() != null) {
            mPeriod = getArguments().getString(ARG_PERIOD);
            //TODO: Get references to the layout and update elements

        }
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {

        // Inflate the layout for this fragment
        View rootView =  inflater.inflate(R.layout.fragment_classification, container, false);

        // Get references to layout elements
        mProgressBar = rootView.findViewById(R.id.cl_progress);
        mContent = rootView.findViewById(R.id.cl_content);
        // Get the reference to the recyclerView in the layout
        mRecyclerView = rootView.findViewById(R.id.classification_list);
        // Get the reference to the SwipeRefreshLayout and set a listener to call the action
        // that has to be done when swiping
        mSwipeRefreshLayout = rootView.findViewById(R.id.cl_swipeRefreshLayout);
        mSwipeRefreshLayout.setColorSchemeColors(getActivity().getResources().getColor(R.color.colorPrimary), getActivity().getResources().getColor(R.color.light_primary_color));

        mSwipeRefreshLayout.setOnRefreshListener(new SwipeRefreshLayout.OnRefreshListener() {
            @Override
            public void onRefresh() {
                // Code to be executed when swipe is performed
                swipePerformed = TRUE;
                readClassification();
            }

        });


        readClassification();

        return rootView;

    }

    private void readClassification () {

        //Hide content and show the progress bar
        showLoading();
        // Get an instance of FireStore
        FirebaseFirestore fireStoreDB = FirebaseFirestore.getInstance();

        switch (mPeriod) {
            case ClassificationPageAdapter.TOTAL:

                //Get a reference to user Collection
                CollectionReference usersCollectionReference = fireStoreDB.collection("users");
                //Execute query to get user's global stats
                usersCollectionReference
                        .orderBy("totalDistance", Query.Direction.DESCENDING)
                        .orderBy("totalTime", Query.Direction.DESCENDING)
                        .get()
                        .addOnCompleteListener(task -> {
                            if (task.isSuccessful()) {
                                ArrayList<CaminaUser> caminaUsers = new ArrayList<>();
                                for (QueryDocumentSnapshot document : task.getResult())
                                {
                                caminaUsers.add (document.toObject(CaminaUser.class));
                                }


                                // Finally, setup Recycle View
                                setupRecyclerView(caminaUsers);
                                // And show it
                                showClassification();
                            }

                            else {
                                mProgressBar.setVisibility(View.INVISIBLE);
                                Log.e("****************RECORDS", "Error getting documents: ", task.getException());
                                Toast.makeText(this.getActivity(),
                                        R.string.show_records_error + task.getException().getMessage(),
                                        Toast.LENGTH_LONG).show();
                            }
                            });


                break;

            case ClassificationPageAdapter.MONTH:

                Calendar calendar = Calendar.getInstance();
                int year = calendar.get(Calendar.YEAR);
                int month = calendar.get(Calendar.MONTH) + 1; //Calendar.MONTH is zero based, so it is giving 1 less than current month
                // We use a collection group, to retrieve results from a collection group consisting of all collections with the same ID
                // (even though they might collections from different parents, i.e. different users in our case.
                fireStoreDB.collectionGroup("monthlyRegistries") //We will retrieve all documents belonging to collections monthlyRegistries
                        .whereEqualTo("year", year) // of this year
                        .whereEqualTo("month", month) // and this month
                        .orderBy("distance", Query.Direction.DESCENDING)
                        .orderBy("time", Query.Direction.DESCENDING)
                        .get()
                        .addOnSuccessListener((QuerySnapshot queryDocumentSnapshots) -> {
                            final int totalRegistries = queryDocumentSnapshots.size();
                            // Due to the asynchronous nature of queries, we can't use a list to store
                            // the users whom registries belong. We should use a HashMap, and store its position
                            HashMap <Integer, CaminaUser> CaminaUsersHash = new HashMap<>();
                            // In this list we will store the ordered monthly registries, to be able to retrieve its position
                            ArrayList <QueryDocumentSnapshot> allMonthlyRegistries = new ArrayList<>();

                            for (QueryDocumentSnapshot monthlyRegistry : queryDocumentSnapshots)
                            {
                                allMonthlyRegistries.add (monthlyRegistry); //This list contains the registries of this month
                                                                            //ordered by distance and time
                            }

                            for (QueryDocumentSnapshot monthlyRegistry : queryDocumentSnapshots)
                            {
                                MonthlyStatistics monthlyStatistics = monthlyRegistry.toObject(MonthlyStatistics.class);
                                //We get the user whom this monthly registry belongs
                                monthlyRegistry.getReference().getParent().getParent()
                                        .get()
                                        .addOnCompleteListener(task -> {
                                        if (task.isSuccessful()) {
                                            CaminaUser owner = task.getResult().toObject(CaminaUser.class);
                                            owner.setTotalSkippedStations(monthlyStatistics.getSkippedStations());
                                            owner.setTotalCalories(monthlyStatistics.getCalories());
                                            owner.setTotalDistance(monthlyStatistics.getDistance());
                                            owner.setTotalTime(monthlyStatistics.getTime());
                                            CaminaUsersHash.put (allMonthlyRegistries.indexOf(monthlyRegistry), owner);
                                            if (CaminaUsersHash.size() == totalRegistries) {

                                                // Finally, setup Recycle View
                                                setupRecyclerView(new ArrayList <CaminaUser>(CaminaUsersHash.values()));
                                                // And show it
                                                showClassification();

                                            }
                                        }
                                        else {
                                            mProgressBar.setVisibility(View.INVISIBLE);
                                            Log.e("****************RECORDS", "Error getting documents: ", task.getException());
                                            Toast.makeText(getActivity(),
                                                "No se ha podido recuperar la clasificación mensual" + task.getException().getMessage(),
                                                Toast.LENGTH_LONG).show();
                                    }


                                });


                            }
                            // ...
                        })
                        .addOnFailureListener(e -> {
                            mProgressBar.setVisibility(View.INVISIBLE);
                            Log.e("****************RECORDS", "Error getting documents: ", e);
                            Toast.makeText(getActivity(),
                                    R.string.show_records_error + e.getMessage(),
                                    Toast.LENGTH_LONG).show();

                        });
                 break;

        }
    }





    @Override
    public void onAttach(Context context) {
        super.onAttach(context);
        if (context instanceof OnFragmentInteractionListener) {
            mListener = (OnFragmentInteractionListener) context;
        } else {
            throw new RuntimeException(context.toString()
                    + " must implement OnFragmentInteractionListener");
        }
    }

    @Override
    public void onDetach() {
        super.onDetach();
        mListener = null;
    }

    /**
     * This interface must be implemented by activities that contain this
     * fragment to allow an interaction in this fragment to be communicated
     * to the activity and potentially other fragments contained in that
     * activity.
     */
    public interface OnFragmentInteractionListener {
        // TODO: Update argument type and name
        void onFragmentInteraction(Uri uri);
    }

    private void showLoading() {
        //Show progressbar and hide content
        if (!swipePerformed) {
            mProgressBar.setVisibility(View.VISIBLE);
            mContent.setVisibility(View.GONE);
        }
    }

    private void showClassification() {

        //Show content and hide progressbar
        mProgressBar.setVisibility(View.GONE);
        mContent.setVisibility(View.VISIBLE);
        if (swipePerformed) {
            mSwipeRefreshLayout.setRefreshing(false);
            swipePerformed = false;
        }
    }

    public void setupRecyclerView (List<CaminaUser> caminaUsers) {

        //We create and set a suitable adapter for the recyclerView
        mAdapter = new SimpleItemRecyclerViewAdapter(this, caminaUsers);
        mRecyclerView.setAdapter(mAdapter);

    }


    /************************************************ RECYCLER VIEW ****************************************************************************/

    public class SimpleItemRecyclerViewAdapter extends RecyclerView.Adapter<ClassificationFragment.SimpleItemRecyclerViewAdapter.ViewHolder> {


        private final ClassificationFragment mParentActivity; //The parent
        private List<CaminaUser> mCaminaUsers; //The items to display in the RecyclerView (Metro Stops)

        //The constructor

        SimpleItemRecyclerViewAdapter(ClassificationFragment parent, List<CaminaUser> caminaUsers) {

            mParentActivity = parent;
            mCaminaUsers = caminaUsers;

        }

        /**
         * This method creates different RecyclerView.ViewHolder objects based on the item's position
         * odd or even
         *
         * @param parent   ViewGroup container for the item
         * @param viewType Type of view to be inflated
         * @return inflated viewHolder
         */

        @Override
        public SimpleItemRecyclerViewAdapter.ViewHolder onCreateViewHolder(ViewGroup parent, int viewType) {

            View view;
            LayoutInflater inflater = LayoutInflater.from(parent.getContext());
            int layoutResource = 0; // Will store the assigned layout id, depending on the item position

            switch (viewType) {

                case ODD:
                    layoutResource = R.layout.item_classification_even;

                    break;

                case EVEN:
                    layoutResource = R.layout.item_classification_odd;
                    break;
            }

            //The view is inflated with the appropriate layout and then returned
            view = inflater.inflate(layoutResource, parent, false);
            return new SimpleItemRecyclerViewAdapter.ViewHolder(view);

        }


        //Returns the view type of the item at position for the purpose of view recycling.
        @Override
        public int getItemViewType(int position) {
            int type;
            type = position % 2; //Module2 is calculated to know whether it's odd or even
            if (type == 0) return (ODD);
            else return (EVEN);

        }

        // Return the size of the dataset (invoked by the layout manager)
        @Override
        public int getItemCount() {

            return mCaminaUsers.size();
        }

        class ViewHolder extends RecyclerView.ViewHolder {

            final TextView mCompleteName;
            final TextView mSkippedStations;
            final TextView mCalories;
            final TextView mDistance;
            final TextView mTime;
            final ImageView mUserPicture;

            // Create a constructor that accepts the entire item row
            // and does the view lookups to find each subview
            ViewHolder(View view) {
                super(view);
                //Set the corresponding subviews of view to ViewHolder views
                mCompleteName = view.findViewById (R.id.cl_textView_name);
                mSkippedStations = view.findViewById(R.id.cl_textView_metro_stops);
                mCalories = view.findViewById(R.id.cl_textView_calories);
                mDistance = view.findViewById(R.id.cl_textView_distance);
                mTime = view.findViewById(R.id.cl_textView_time);
                mUserPicture = view.findViewById(R.id.cl_imageView_user_picture);
            }

        }

        /**
         * This method internally calls onBindViewHolder(ViewHolder, int) to update the
         * RecyclerView.ViewHolder contents with the item at the given position
         * and also sets up some private fields to be used by RecyclerView.
         *
         * @param holder   The type of RecyclerView.ViewHolder to populate
         * @param position Item position in the viewGroup.
         */

        @Override
        public void onBindViewHolder(final SimpleItemRecyclerViewAdapter.ViewHolder holder, final int position) {

            // The given position determines the corresponding CaminaUser
            // at the same position in List<CaminaUser> mCaminaUsers
            // Set holder views with the corresponding values

            holder.mCompleteName.setText(mCaminaUsers.get(position).getCompleteName());
            holder.mSkippedStations.setText(getString(R.string.cl_metro_stops,mCaminaUsers.get(position).getTotalSkippedStations()));
            holder.mCalories.setText(getString(R.string.cl_calories, mCaminaUsers.get(position).getTotalCalories()));
            holder.mDistance.setText(Utitilies.FormatDistanceDoubleToString((double)mCaminaUsers.get(position).getTotalDistance()));
            holder.mTime.setText(Utitilies.FormatTimeIntegerToString(mCaminaUsers.get(position).getTotalTime()));


            Picasso.get()
                    .load(mCaminaUsers.get(position).getPictureUrl())
                    .resize(300, 300)
                    .centerCrop()
                    .placeholder(R.drawable.man)
                    .error(R.drawable.man)
                    .into(holder.mUserPicture);

        }

    }


}
