package jimenezcarlos.android.uoc.edu.camina;

import android.app.AlertDialog;
import android.content.Context;
import android.content.Intent;
import android.graphics.Color;
import android.graphics.PorterDuff;
import android.os.Bundle;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.ProgressBar;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.appcompat.app.AppCompatActivity;
import androidx.appcompat.widget.Toolbar;
import androidx.cardview.widget.CardView;
import androidx.recyclerview.widget.RecyclerView;

import com.google.gson.Gson;
import com.google.gson.JsonObject;
import com.google.gson.reflect.TypeToken;

import java.io.IOException;
import java.lang.reflect.Type;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import jimenezcarlos.android.uoc.edu.menu.LateralMenu;
import jimenezcarlos.android.uoc.edu.rest.adapter.TMBAdapter;
import jimenezcarlos.android.uoc.edu.rest.model.Path;
import jimenezcarlos.android.uoc.edu.rest.model.TMBPlannerAPI.Itinerary;
import jimenezcarlos.android.uoc.edu.rest.model.TMBPlannerAPI.Leg;
import jimenezcarlos.android.uoc.edu.rest.model.TMBPlannerAPI.Point;
import jimenezcarlos.android.uoc.edu.rest.model.TMBPlannerAPI.SerializableOriginDestination;
import jimenezcarlos.android.uoc.edu.rest.model.TMBPlannerAPI.TransitStop;
import jimenezcarlos.android.uoc.edu.rest.model.TMBPlannerAPI.Plan;
import jimenezcarlos.android.uoc.edu.util.LineToTransitIconDrawable;
import jimenezcarlos.android.uoc.edu.util.Utitilies;
import retrofit2.Call;
import retrofit2.Callback;
import retrofit2.Response;

import static android.view.View.VISIBLE;
import static android.view.View.INVISIBLE;
import static java.lang.Boolean.FALSE;
import static jimenezcarlos.android.uoc.edu.camina.RouteSelectionActivity.Epoch2DateString;
import static jimenezcarlos.android.uoc.edu.util.Utitilies.FormatDistanceDoubleToString;
import static jimenezcarlos.android.uoc.edu.util.Utitilies.FormatTimeIntegerToString;

public class MetroStopSelectionActivity extends AppCompatActivity {


    private static final String TAG = "RouteSelectionActivity";

    // Reference to the recyclerView
    private RecyclerView recyclerView;
    // This is the adapter to the RecyclerView
    private SimpleItemRecyclerViewAdapter adapter;
    // Reference to the progress bar
    private ProgressBar mProgressBar;
    // More references to layout items
    // For the selected route viewCard
    private TextView mWalkingTime;
    private TextView mWalkingDistance;
    private TextView mTotalTime;
    private ImageView[] mTransitIcon = new ImageView[4];
    // For the rest of information
    private TextView mOriginAddress;
    private ImageView mFirstLegTransitIcon;
    private TextView mDistanceAndTimeToFirstMetroStation;
    private TextView mDirectionIndication;
    private ImageView mVerticalLine;
    private int auxCounter;




    // To identify a the first Metro stop in the list
    private final static int FIRST = 0;
    // To identify a Metro stop in an odd position in the list
    private final static int ODD = 1;
    // To identify a Metro stop in an even position in the list
    private final static int EVEN = 2;

    // Auxiliary class to get the name of drawable resources of transit icon lines
    private LineToTransitIconDrawable iconLines;

    private SerializableOriginDestination originDestination;
    private Plan plan;
    private Integer chosenItineraryId;
    private Itinerary chosenItinerary;
    private List<Leg> legs = new ArrayList<>();
    private List<TransitStop> metroStops = new ArrayList<>();
    private int firstMetroLeg;
    private int colorFirstMetroLine = Color.parseColor("#000000");
    private int previousMetroStopClicked = -1;

    private int newTotalTime;
    private int newWalkingTime;
    private double newWalkingDistance;

    // Due to the asynchronous nature of queries, we can't use a list to store the values
    // because we could'nt order them properly. We should use a HashMap, and store its position
    private Map<Integer, Integer> walkingTimeToMetroStopsInFirstMetroLeg = new HashMap<>();
    private Map<Integer, Double> walkingDistanceToMetroStopsInFirstMetroLeg = new HashMap<>();

    // The argument which identifies the selected itinerary to be passed to Metro StopSelectionActivity
    public static final String ARG_FINAL_PATH= "final_path";
    public static final String ARG_SKIPPED_ALL_STOPS_IN_FIRST_METRO_LINE= "skipped_all";


    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        // Set the layout for this activity
        setContentView(R.layout.activity_metro_stop_selection);

        // Get the reference to the support action bar in the layout and set it
        Toolbar toolbar = findViewById(R.id.toolbar);
        setSupportActionBar(toolbar);

        //Include the Lateral Menu
        LateralMenu.getDrawer(this,savedInstanceState, toolbar,2);

        // Get the transferred data from RouteSelectionActivity activity.
        Intent intent = getIntent();
        originDestination = (SerializableOriginDestination) intent.getSerializableExtra(RouteSelectionActivity.ARG_SERIALIZABLE_ORIGIN_DESTINY);
        plan = (Plan) intent.getSerializableExtra(RouteSelectionActivity.ARG_PLAN_OBJECT);
        chosenItineraryId = (Integer) intent.getSerializableExtra(RouteSelectionActivity.ARG_SELECTED_ITINERARY);
        chosenItinerary = plan.getItineraries().get(chosenItineraryId);
        legs = chosenItinerary.getLegs();
        firstMetroLeg = chosenItinerary.getFirstMetroLeg();
        metroStops = legs.get(firstMetroLeg).getAllMetroStops();
        // Get the reference to the recyclerView in the layout
        recyclerView = findViewById(R.id.station_list);
        // Get the reference to the progress bar
        mProgressBar = findViewById(R.id.stations_progress);
        // Get the reference to the vertical transit line and make it invisible
        mVerticalLine=  findViewById(R.id.transitLine);
        mVerticalLine.setVisibility(INVISIBLE);
        // Call the method to perform all necessary queries to TMB API
        auxCounter = 0;
        CalculateWalkingParametersToMetroStops();
        //Instantiate a new object of class LineToTransitIconDrawable
        iconLines = new LineToTransitIconDrawable();
        //And show the information of the selected route
        SetInformationInSelectedRoute ();


    }

    public class SimpleItemRecyclerViewAdapter extends RecyclerView.Adapter<SimpleItemRecyclerViewAdapter.ViewHolder> {


        private final MetroStopSelectionActivity mParentActivity; //The parent activity
        private List<TransitStop> mMetroStops; //The items to display in the RecyclerView (Metro Stops)

        //The constructor
        SimpleItemRecyclerViewAdapter (MetroStopSelectionActivity parent, List<TransitStop> metroStops) {

            mParentActivity = parent;
            mMetroStops = metroStops;

        }


        /**
         * This method creates different RecyclerView.ViewHolder objects based on the item's position
         * odd or even
         *
         * @param parent   ViewGroup container for the item
         * @param viewType Type of view to be inflated
         * @return inflated viewHolder
         */

        @Override
        public SimpleItemRecyclerViewAdapter.ViewHolder onCreateViewHolder(ViewGroup parent, int viewType) {

            View view;
            LayoutInflater inflater = LayoutInflater.from(parent.getContext());
            int layoutResource = 0; // Will store the assigned layout id, depending on the item position

            switch (viewType) {

                case FIRST:

                    layoutResource = R.layout.item_list_metro_station_first_one;
                    break;

                default:
                    layoutResource = R.layout.item_list_metro_station;

                    break;

            }

            //The view is inflated with the appropriate layout and then returned
            view = inflater.inflate(layoutResource, parent, false);
            return new SimpleItemRecyclerViewAdapter.ViewHolder(view);

        }


        //Returns the view type of the item at position for the purpose of view recycling.
        @Override
        public int getItemViewType(int position) {

            if (position==0) {
                return (FIRST);
            }
            else {
                int type;
                type = position % 2; //Module2 is calculated to know whether it's odd or even
                if (type == 0) return (ODD);
                else return (EVEN);
            }

        }


        /**
         * This method internally calls onBindViewHolder(ViewHolder, int) to update the
         * RecyclerView.ViewHolder contents with the item at the given position
         * and also sets up some private fields to be used by RecyclerView.
         *
         * @param holder   The type of RecyclerView.ViewHolder to populate
         * @param position Item position in the viewGroup.
         */

        @Override
        public void onBindViewHolder(final SimpleItemRecyclerViewAdapter.ViewHolder holder, final int position) {

            // The given position determines the corresponding Metro station
            // at the same position in List<TransitStop> metroStops
            // Set holder views with the corresponding values

            if (position == previousMetroStopClicked) { //If the card corresponds to the one selected, make it green.

                holder.mOuterCardView.setCardBackgroundColor(getResources().getColor(R.color.healthiestItineraryBorderColor));
                holder.mInnerCardView.setCardBackgroundColor(getResources().getColor(R.color.healthiestItineraryBackgroundColor));
            } else { //If not, make it gray

                holder.mOuterCardView.setCardBackgroundColor(getResources().getColor(R.color.itineraryBorderColor));
                holder.mInnerCardView.setCardBackgroundColor(getResources().getColor(R.color.itineraryBackgroundColor));

            }



            holder.mStationName.setText(mMetroStops.get(position).getName()); // Set the name of the Metro stop
            holder.mPointIcon.setColorFilter(colorFirstMetroLine, PorterDuff.Mode.MULTIPLY); // And tint the stop icon with the corresponding color of Metro line

            if (position > 0) { //For all metro stops except the first one (index 0)

                if (walkingTimeToMetroStopsInFirstMetroLeg.get(position)>=0) { // If we could get walking distance and walking time to it (if not, value of this filed is -1)...
                    // Set values of walking distance and walking time to every Metro Station's view holder
                    Double extraDistanceWalking = walkingDistanceToMetroStopsInFirstMetroLeg.get(position) - chosenItinerary.getWalkingDistanceToFirstMetroStation();
                    holder.mExtraDistanceWalking.setText(Utitilies.FormatDistanceDoubleToString(extraDistanceWalking));
                    int extraTimeWalking = walkingTimeToMetroStopsInFirstMetroLeg.get(position) - chosenItinerary.getWalkingTimeToFirstMetroStation();
                    holder.mExtraTimeWalking.setText(Utitilies.FormatTimeIntegerToString(extraTimeWalking));
                    //TODO: Change '70' for the actual user's weight
                    double extraCalories = Math.ceil(0.029 * 70 * 2.2 * extraTimeWalking / 60);
                    holder.mExtraCaloriesConsumed.setText(String.format("%.0f", extraCalories) + " cal");  //No decimals
                    holder.mTooFarMessage.setVisibility(INVISIBLE);
                    holder.mIconClock.setVisibility(VISIBLE);
                    holder.mIconFlame.setVisibility(VISIBLE);
                    holder.mIconWalk.setVisibility(VISIBLE);
                    holder.mPlusSign_1.setVisibility(View.VISIBLE);
                    holder.mPlusSign_2.setVisibility(View.VISIBLE);
                    holder.mPlusSign_3.setVisibility(View.VISIBLE);
                    holder.mExtraTimeWalking.setVisibility(VISIBLE);
                    holder.mExtraDistanceWalking.setVisibility(VISIBLE);
                    holder.mExtraCaloriesConsumed.setVisibility(VISIBLE);
                }
                else {
                    // TMBTransitAPI considers the distance to this Metro Station is too high to make it by foot, so doesn't give an answer
                    // As an approach to solve this problem, I've decided to give this information to the user and make this station not selectable
                    holder.mTooFarMessage.setVisibility(VISIBLE); //Only make visible this message to inform the user
                    holder.mIconClock.setVisibility(INVISIBLE);
                    holder.mIconFlame.setVisibility(INVISIBLE);
                    holder.mIconWalk.setVisibility(INVISIBLE);
                    holder.mPlusSign_1.setVisibility(View.INVISIBLE);
                    holder.mPlusSign_2.setVisibility(View.INVISIBLE);
                    holder.mPlusSign_3.setVisibility(View.INVISIBLE);
                    holder.mExtraTimeWalking.setVisibility(INVISIBLE);
                    holder.mExtraDistanceWalking.setVisibility(INVISIBLE);
                    holder.mExtraCaloriesConsumed.setVisibility(INVISIBLE);
                    holder.itemView.setFocusable(FALSE);
                    holder.itemView.setClickable(FALSE);

                }

            }



            /***************************** MANAGING CLICKS ON METRO STATIONS  ********************************/

            // A new onClickListener is created and set to the view of ViewHolder
            holder.itemView.setOnClickListener(v -> { //When clicked
                // Obtain context of this Activity
                final Context context = v.getContext();
                int currentPos;
                // Obtain the position of clicked Metro Station
                currentPos = holder.getAdapterPosition();
                // Call the method to show the book's details, passing the title and the view

                if (currentPos == previousMetroStopClicked) {//If it's the second time in a row that this adapter has ben clicked

                    //Create an object of class Path to pass to StartItineraryActivity

                    Path path = new Path ();
                    path.setOriginAddress(originDestination.getOriginAddress());
                    path.setDestinationAddress(originDestination.getDestinationAddress());
                    path.setMetroLines(chosenItinerary.getMetroLines());
                    path.setMetroLineColors(chosenItinerary.getMetroLineColors());
                    path.setStationNames(chosenItinerary.getStationNames());
                    path.setLatFirstMetroStop(metroStops.get(currentPos).getLat().toString());
                    path.setLonFirstMetroStop(metroStops.get(currentPos).getLon().toString());
                    path.setStartWalkingDistance(walkingDistanceToMetroStopsInFirstMetroLeg.get(currentPos));
                    path.setStartWalkingTime(walkingTimeToMetroStopsInFirstMetroLeg.get(currentPos));
                    path.setEndWalkingDistance(chosenItinerary.getWalkingDistanceFromLastMetroStationToDestination());
                    path.setEndWalkingTime(chosenItinerary.getWalkingTimeFromLastMetroStationToDestination());
                    path.setSkippedStations(currentPos);
                    path.setWalkDistance(newWalkingDistance);
                    path.setWalkTime(newWalkingTime);
                    //TODO: Get actual user's weight. For now, 70 kg is taken for everybody
                    double totalCalories = Math.ceil(0.029 * 70 * 2.2 * newWalkingTime / 60);
                    path.setCalories(totalCalories);
                    path.setOMSActivityRatio((double)newWalkingTime/60/43*100); //OMS recommends 300 minutes a week, so 43 minutes a day
                    Intent intent = new Intent(context, StartItineraryActivity.class);

                    if (currentPos==(metroStops.size()-1)) { //If the user has decided to skip ALL the metro stops in the FIRST Metro Line

                        path.deleteFirstMetroLine(); // Remove from path the first Metro line of all properties
                        path.setSkippedAll(true);

                        // Set as first Metro station to reach on foot, the first Metro station of the second transit line
                        // This is necessary because sometimes the connections between Metro lines aren't in the same Metro Station.
                        // In those cases, it's necessary to walk from the last Metro station in the first transit line to the
                        // first Metro station in the second transit line
                        int secondMetroLeg = chosenItinerary.getSecondMetroLeg();
                        path.setLatFirstMetroStop(legs.get(secondMetroLeg).getAllMetroStops().get(0).getLat().toString());
                        path.setLonFirstMetroStop(legs.get(secondMetroLeg).getAllMetroStops().get(0).getLon().toString());
                        intent.putExtra(ARG_SKIPPED_ALL_STOPS_IN_FIRST_METRO_LINE, true);

                    }

                    intent.putExtra(ARG_FINAL_PATH, path);
                    context.startActivity(intent);

                }

                else //********* It's the first time in a raw this Metro Station is clicked *************//

                {
                    if (walkingTimeToMetroStopsInFirstMetroLeg.get(currentPos)>=0)

                    { //Those stations which TMB considers that are too far, are flagged with '-1'
                        //and thus are not selectable

                        if (currentPos == (metroStops.size() - 1) && chosenItinerary.getTotalMetroLegs() == 1)
                        // If the itinerary includes JUST ONE Metro Line, the user cannot skip the whole Metro Line,
                        // because ONLY WALK routes are NOT permitted in Camina.
                        // Inform the user

                        {
                            AlertDialog alertDialog = new AlertDialog.Builder(MetroStopSelectionActivity.this).create();
                            alertDialog.setTitle(R.string.only_walk_itinerary_title);
                            alertDialog.setMessage(getString(R.string.only_walk_itinerary_message));
                            alertDialog.setButton(AlertDialog.BUTTON_NEUTRAL, "OK",
                                    (dialog, which) -> {
                                        dialog.dismiss();

                                    });
                            alertDialog.show();


                        }

                        else
                        {   //Manage this first click

                            previousMetroStopClicked = currentPos;

                            // Calculate new total time, walking time and walking distance
                            if (currentPos == 0) //To avoid little inconsistencies in values shown, due to different information given by TMBPlannerAPI
                            //We make sure when the first Metro stop is selected, the information shown is the same as that shown at the beginning
                            {

                                newTotalTime = chosenItinerary.getDuration();
                                newWalkingTime = chosenItinerary.getWalkingTimeAddingAllLegs();
                                newWalkingDistance = chosenItinerary.getWalkingDistanceAddingAllLegs();

                            }

                            else if (currentPos == (metroStops.size() - 1))

                            {  // If user selects the last metro stop and, thus, skip the whole first metro leg

                                // The new total time is the WALKING time from origin to chosen Metro stop
                                // + chosen itinerary's duration from the following leg's first step until the end
                                //newTotalTime = walkingTimeToMetroStopsInFirstMetroLeg.get(currentPos) + chosenItinerary.getItineraryDurationFromGivenLegToEnd (chosenItinerary.getFirstMetroLeg()+1);
                                newTotalTime = walkingTimeToMetroStopsInFirstMetroLeg.get(currentPos) + chosenItinerary.getItineraryDurationFromStartToGivenLeg (chosenItinerary.getFirstMetroLeg()+1);
                                newWalkingTime = chosenItinerary.getWalkingTimeAddingAllLegs() - chosenItinerary.getWalkingTimeToFirstMetroStation() + walkingTimeToMetroStopsInFirstMetroLeg.get(currentPos);
                                newWalkingDistance = chosenItinerary.getWalkingDistanceAddingAllLegs() - chosenItinerary.getWalkingDistanceToFirstMetroStation() + walkingDistanceToMetroStopsInFirstMetroLeg.get(currentPos);


                            }

                            else

                                // The new total time is the chosen itinerary duration
                                // + walking time from origin to chosen Metro stop
                                // - chosen itinerary's duration from origin to chosen transit stop
                                newTotalTime = chosenItinerary.getDuration() + walkingTimeToMetroStopsInFirstMetroLeg.get(currentPos) - chosenItinerary.getItineraryDurationFromOriginToChosenMetroStop (currentPos);
                                newWalkingTime = chosenItinerary.getWalkingTimeAddingAllLegs() - chosenItinerary.getWalkingTimeToFirstMetroStation() + walkingTimeToMetroStopsInFirstMetroLeg.get(currentPos);
                                newWalkingDistance = chosenItinerary.getWalkingDistanceAddingAllLegs() - chosenItinerary.getWalkingDistanceToFirstMetroStation() + walkingDistanceToMetroStopsInFirstMetroLeg.get(currentPos);

                            {

                            }
                            //Update information in Selected Route
                            mTotalTime.setText(Utitilies.FormatTimeIntegerToString(newTotalTime));
                            mWalkingTime.setText(Utitilies.FormatTimeIntegerToString(newWalkingTime));
                            mWalkingDistance.setText(Utitilies.FormatDistanceDoubleToString(newWalkingDistance));
                            notifyDataSetChanged();

                            //Ask for confirmation
                            TextView mMessage = findViewById(R.id.textView_which_metro_station);
                            if (currentPos == (metroStops.size() - 1)) //If the user has decided to skip ALL the metro stops in the first Metro Line
                                mMessage.setText(getString(R.string.confirmation_message_skip_all, legs.get(firstMetroLeg).getRoute()));
                            else
                                mMessage.setText(getString(R.string.confirmation_message, metroStops.get(position).getName()));
                        }
                    }
                }

            });

        }

        // Return the size of the dataset (invoked by the layout manager)
        @Override
        public int getItemCount() {

            return mMetroStops.size();
        }

        class ViewHolder extends RecyclerView.ViewHolder {

            final TextView mStationName;
            final TextView mExtraTimeWalking;
            final TextView mExtraDistanceWalking;
            final TextView mExtraCaloriesConsumed;
            final ImageView mPointIcon;

            final CardView mOuterCardView;
            final CardView mInnerCardView;

            final TextView mPlusSign_1;
            final TextView mPlusSign_2;
            final TextView mPlusSign_3;
            final ImageView mIconClock;
            final ImageView mIconFlame;
            final ImageView mIconWalk;
            final TextView mTooFarMessage;


            // Create a constructor that accepts the entire item row
            // and does the view lookups to find each subview
            ViewHolder(View view) {
                super(view);
                //Set the corresponding subviews of view to ViewHolder views
                mStationName = view.findViewById (R.id. textView_station_name);
                mExtraTimeWalking = view.findViewById(R.id.textView_extra_time_walking);
                mExtraDistanceWalking = view.findViewById(R.id.textView_extra_distance);
                mExtraCaloriesConsumed = view.findViewById(R.id.textView_extra_calories);
                mPointIcon = view.findViewById(R.id.imageView_icon_point);
                mOuterCardView = view.findViewById(R.id.outerCardView);
                mInnerCardView = view.findViewById(R.id.innerCardView);
                mIconClock = view.findViewById(R.id.imageView_icon_clock);
                mIconFlame = view.findViewById(R.id.imageView_icon_flame);
                mIconWalk = view.findViewById(R.id.imageView_icon_walk);
                mPlusSign_1 = view.findViewById(R.id.textView_suma_1);
                mPlusSign_2 = view.findViewById(R.id.textView_suma_2);
                mPlusSign_3 = view.findViewById(R.id.textView_suma_3);
                mTooFarMessage = view.findViewById(R.id.textView_TooFar);

            }
            void SetItemsInvisible () {

                mIconClock.setVisibility(INVISIBLE);
                mIconFlame.setVisibility(INVISIBLE);
                mIconWalk.setVisibility(INVISIBLE);
                mPlusSign_1.setVisibility(View.INVISIBLE);
                mPlusSign_2.setVisibility(View.INVISIBLE);
                mPlusSign_3.setVisibility(View.INVISIBLE);
                mExtraTimeWalking.setVisibility(INVISIBLE);
                mExtraDistanceWalking.setVisibility(INVISIBLE);
                mExtraCaloriesConsumed.setVisibility(INVISIBLE);
                mTooFarMessage.setVisibility(INVISIBLE);

            }


        }

    }

    private void setupRecyclerView() {

        adapter = new MetroStopSelectionActivity.SimpleItemRecyclerViewAdapter(this, metroStops);
        recyclerView.setAdapter(adapter);

    }

    private void SetInformationInSelectedRoute (){

        mWalkingTime = findViewById (R.id.text_time_walking);
        mWalkingDistance = findViewById(R.id.text_distance_walking);
        mTotalTime = findViewById(R.id.text_total_time_to_destination);
        mOriginAddress = findViewById(R.id.textOrigin);
        mDistanceAndTimeToFirstMetroStation = findViewById(R.id.textView_walk_indications);

        mTotalTime.setText(Utitilies.FormatTimeIntegerToString(chosenItinerary.getDuration()));
        mWalkingDistance.setText(Utitilies.FormatDistanceDoubleToString(chosenItinerary.getWalkingDistanceAddingAllLegs()));
        mWalkingTime.setText(Utitilies.FormatTimeIntegerToString(chosenItinerary.getWalkingTimeAddingAllLegs()));
        mOriginAddress.setText("Origen: " + originDestination.getOriginAddress());


        // Get references to the transit icons in the Selected Route cardView
        for (int i=0; i<4; i++) {
            String resource = "image_transit_line_icon_"+ String.valueOf(i+1);
            int id = getResources().getIdentifier(resource, "id", getBaseContext().getPackageName());
            mTransitIcon[i] = findViewById(id);
        }


        int i= 0;
        int metroLegsCounter = 0;
        for (Leg l : legs )

        {

            // In order to draw the icons of all the Metro Lines in the chosen itinerary...
            if (l.getMode().equals("SUBWAY")) {

                //For each SUBWAY leg, get the icon of the Metro line and set it to the imageView
                //First, make the preceding arrow visible

                String resource = "imageView_right_arrow_"+ String.valueOf(metroLegsCounter+1);
                int id = getResources().getIdentifier(resource, "id", getBaseContext().getPackageName());
                findViewById(id).setVisibility(VISIBLE);

                resource = iconLines.getResourceName(l.getRoute());
                id = getResources().getIdentifier(resource, "drawable", getBaseContext().getPackageName());
                mTransitIcon[metroLegsCounter].setImageDrawable(getDrawable(id));
                mTransitIcon[metroLegsCounter].setVisibility(VISIBLE);
                metroLegsCounter++;

                // For the first Metro Leg, we set the Direction indication and the big transit line icon
                if (i==firstMetroLeg) {

                    mFirstLegTransitIcon = findViewById(R.id.iconTransitLine);
                    mFirstLegTransitIcon.setImageDrawable(getDrawable(id));

                    mDirectionIndication = findViewById(R.id.textView_direction_indication);
                    mDirectionIndication.setText(legs.get(i).getHeadsign());


                }
            }


            i++;

        }

        // Set the indications to the first Metro Stop
        mDistanceAndTimeToFirstMetroStation.setText (
                getString (R.string.text_walking_indications, FormatDistanceDoubleToString(chosenItinerary.getWalkingDistanceToFirstMetroStation())
                        ,FormatTimeIntegerToString(chosenItinerary.getWalkingTimeToFirstMetroStation())));


    }

    private void ParseTMBPlannerAPIResponse(JsonObject responseTMBPlannerAPI, Plan plan) {

        // Convert and parse the date of API call
        plan.setDate(Epoch2DateString(responseTMBPlannerAPI.getAsJsonObject("plan").getAsJsonPrimitive("date").getAsString()));
        // We will use a Gson object to parse some objects into classes automatically
        Gson g = new Gson();
        // Parse JSON objects from and to into Point objects
        Point point = g.fromJson(responseTMBPlannerAPI.getAsJsonObject("plan").getAsJsonObject("from"), Point.class);
        plan.setFrom(point);
        point = g.fromJson(responseTMBPlannerAPI.getAsJsonObject("plan").getAsJsonObject("to"), Point.class);
        plan.setTo(point);

        // Gson class TypeToken is used to assist us in finding the correct Type for our class configuration when trying to get a List of objects of that class
        Type itineraryListType = new TypeToken<ArrayList<Itinerary>>(){}.getType();
        List<Itinerary> itineraries = g.fromJson(responseTMBPlannerAPI.getAsJsonObject("plan").getAsJsonArray("itineraries"), itineraryListType);

        // Add the list of itineraries to the plan object
        plan.setItineraries(itineraries);

    }

    private void CalculateWalkingParametersToMetroStops (){

        // For every Metro stop calculate walking distance and walking time from origin to that particular stop
        // Construct the Map of query parameters to call TMBPlannerAPI

        Map<String, String> TMBPlannerQuery = new HashMap<>();

        mProgressBar.setVisibility(VISIBLE);

        for (TransitStop mS:metroStops) //For every Metro stop, construct the query
        {

            TMBPlannerQuery.put("showIntermediateStops", "FALSE");
            TMBPlannerQuery.put("mode", "WALK");
            TMBPlannerQuery.put("fromPlace", originDestination.getOriginCoordinates());
            TMBPlannerQuery.put("toPlace", mS.getLat() + ", " + mS.getLon());

            Call<JsonObject> callTMBPlannerAPIResponse;
            callTMBPlannerAPIResponse = new TMBAdapter().getPlannedRoute(TMBPlannerQuery);
            callTMBPlannerAPIResponse.enqueue(new Callback<JsonObject>() {

                @Override
                public void onResponse(@NonNull Call<JsonObject> call, @NonNull Response<JsonObject> response) {

                    auxCounter++;

                    JsonObject responseTMBPlannerAPI = response.body();
                    if (responseTMBPlannerAPI != null) {
                        Plan walkingPlan = new Plan();
                        ParseTMBPlannerAPIResponse(responseTMBPlannerAPI, walkingPlan);
                        // Save walking distance and walking time to every Metro Station
                        walkingTimeToMetroStopsInFirstMetroLeg.put(metroStops.indexOf(mS), walkingPlan.getItineraries().get(0).getWalkTime());
                        walkingDistanceToMetroStopsInFirstMetroLeg.put(metroStops.indexOf(mS), walkingPlan.getItineraries().get(0).getWalkDistance());

                    } else {

                        // TMBTransitAPI considers the distance to this Metro Station is too high to make it by foot, so doesn't give an answer
                        // As an approach to solve this problem, I've decided to give this information to the user and make this station not selectable
                        // I'll flag this cases assigning -1
                        walkingTimeToMetroStopsInFirstMetroLeg.put(metroStops.indexOf(mS), -1);
                        walkingDistanceToMetroStopsInFirstMetroLeg.put(metroStops.indexOf(mS), -1.0);

                    }

                    if (auxCounter==metroStops.size()) {
                        // When all the data has been collected, stop the spinner
                        // and setup the recycler view
                        mProgressBar.setVisibility(View.GONE);
                        // Get the reference to the vertical line (transit line) and tint it with the color of the corresponding Metro line (the first one in itinerary)
                        colorFirstMetroLine = Color.parseColor("#"+legs.get(firstMetroLeg).getRouteColor());
                        mVerticalLine.setColorFilter(colorFirstMetroLine, PorterDuff.Mode.MULTIPLY);
                        mVerticalLine.setVisibility(VISIBLE);
                        setupRecyclerView();
                    }
                }

                @Override
                public void onFailure(@NonNull Call<JsonObject> call, @NonNull Throwable t) {
                    mProgressBar.setVisibility(View.GONE);
                    // TODO: If we get an error from server, inform the user
                    if (t instanceof IOException) {
                        Log.d("RouteSelectionActivity", "Network failure :( inform the user and possibly retry");
                    } else {

                        Log.d("RouteSelectionActivity", "Conversion issue");
                    }
                }

            });

        }


    }

}
