package jimenezcarlos.android.uoc.edu.camina;

import android.content.Context;
import android.content.Intent;
import android.location.Address;
import android.location.Geocoder;
import android.location.Location;
import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.Toast;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.app.AppCompatActivity;
import androidx.appcompat.widget.Toolbar;

import com.google.android.gms.common.api.Status;
import com.google.android.gms.location.FusedLocationProviderClient;
import com.google.android.gms.location.LocationServices;
import com.google.android.gms.maps.model.LatLng;
import com.google.android.gms.tasks.OnCompleteListener;
import com.google.android.gms.tasks.Task;
import com.google.android.libraries.places.api.Places;
import com.google.android.libraries.places.api.model.Place;
import com.google.android.libraries.places.api.model.RectangularBounds;
import com.google.android.libraries.places.widget.Autocomplete;
import com.google.android.libraries.places.widget.AutocompleteActivity;
import com.google.android.libraries.places.widget.model.AutocompleteActivityMode;

import java.io.IOException;
import java.util.Arrays;
import java.util.List;
import java.util.Locale;

import jimenezcarlos.android.uoc.edu.menu.LateralMenu;
import jimenezcarlos.android.uoc.edu.rest.model.TMBPlannerAPI.SerializableOriginDestination;

public class NewJourneyActivity extends AppCompatActivity {


    // The argument representing the Object of type SerializableOriginDestination
    public static final String ARG_SERIALIZABLE_ORIGIN_DESTINY= "origin_destiny";

    private static final String TAG = "NewJourneyActivity";
    private static final int AUTOCOMPLETE_REQUEST_CODE = 100;

    private EditText editText_place_to_go;
    private Button button_calculate;
    private Place destinationPlace;


    // The entry point to the Fused Location Provider.
    private FusedLocationProviderClient mFusedLocationProviderClient;


    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_new_jorney);


        Toolbar toolbar = findViewById(R.id.toolbar);
        setSupportActionBar(toolbar);
        //Include the Lateral Menu
        LateralMenu.getDrawer(this,savedInstanceState,toolbar,2);

        // Construct a FusedLocationProviderClient.
        mFusedLocationProviderClient = LocationServices.getFusedLocationProviderClient(this);


        // TODO The Calculate button must be active only when a destination has been added
        button_calculate = findViewById(R.id.buttonCalculate);
        button_calculate.setBackgroundColor(getResources().getColor(R.color.inactive_button));
        button_calculate.setOnClickListener(new View.OnClickListener() {
            public void onClick(View v) {
                // When button CALCULATE is clicked
                // a new intent to start a RouteSelectionActivity is created
                // and information of origin and destination is passed (using a SerializableOriginDestination object)
                final Context context = v.getContext();
                Intent intent = new Intent(context, RouteSelectionActivity.class);
                try {

                        Task<Location> locationResult = mFusedLocationProviderClient.getLastLocation();
                        locationResult.addOnCompleteListener(NewJourneyActivity.this, new OnCompleteListener<Location>() {
                            @Override
                            public void onComplete(@NonNull Task<Location> task) {
                                if (task.isSuccessful()) {
                                    // Store devices position
                                    Location mCurrentLocation = task.getResult();
                                    LatLng myCoordinates = new LatLng(mCurrentLocation.getLatitude(), mCurrentLocation.getLongitude());

                                    SerializableOriginDestination infoToPass = new SerializableOriginDestination(destinationPlace, getAddress (myCoordinates), mCurrentLocation);

                                    Intent intent = new Intent(context, RouteSelectionActivity.class);
                                    intent.putExtra(ARG_SERIALIZABLE_ORIGIN_DESTINY, infoToPass);

                                    context.startActivity(intent);
                                    // overridePendingTransition(R.anim.slide_up, R.anim.stay);


                                } else {
                                    Log.d(TAG, "Current location is null. Using defaults.");
                                    Log.e(TAG, "Exception: %s", task.getException());


                                }
                            }
                        });

                } catch (SecurityException e)  {
                    Log.e("Exception: %s", e.getMessage());
                }







            }
        });
        editText_place_to_go = findViewById(R.id.editText);

        editText_place_to_go.setOnFocusChangeListener(new View.OnFocusChangeListener() {
            @Override
            public void onFocusChange(View view, boolean hasFocus) {
                if (hasFocus) {
                    // Make button not clickable
                    button_calculate.setEnabled(false);
                    onSearchCalled();
                }
            }
        });



        //We recall the Places API Key
        String apiKey = getString(R.string.places_api_key);
        //In place the service is not initialized yet, we do it.
        if (!Places.isInitialized()) {
            Places.initialize(getApplicationContext(), apiKey);
        }



    }

    private String getAddress(LatLng myCoordinates) {

        String mAddress = "";
        Geocoder geocoder = new Geocoder(NewJourneyActivity.this, Locale.getDefault());
        try {
            List<Address> addresses = geocoder.getFromLocation(myCoordinates.latitude, myCoordinates.longitude, 1);
            mAddress = addresses.get(0).getAddressLine(0);

        } catch (IOException e) {
            e.printStackTrace();
        }
        return mAddress;



    }

    @Override
    public void onResume (){

        super.onResume();
        editText_place_to_go.clearFocus(); //We make the editText lose the focus, so that the autocomplete Activity is triggered when the user clicks on it again


    }


    public void onSearchCalled() {
        // Set the fields to specify which types of place data to return.
        List<Place.Field> fields = Arrays.asList(Place.Field.ID, Place.Field.NAME, Place.Field.ADDRESS, Place.Field.LAT_LNG);
        // Start the autocomplete intent.
        Intent intent = new Autocomplete.IntentBuilder(
                AutocompleteActivityMode.FULLSCREEN, fields).setCountry("ES") //Constrain the results to Country: Spain
                .setLocationRestriction(RectangularBounds.newInstance(        //Restrict the results to Barcelona province
                new LatLng(41.166217, 1.436145),
                new LatLng(42.013023, 2.839965)))
                .build(this);
        startActivityForResult(intent, AUTOCOMPLETE_REQUEST_CODE);
    }


    @Override
    protected void onActivityResult(int requestCode, int resultCode, @Nullable Intent data) {
        // This method is called whe Google Places API returns the result
        super.onActivityResult(requestCode, resultCode, data);
        if (requestCode == AUTOCOMPLETE_REQUEST_CODE) {
            if (resultCode == RESULT_OK) {
                // Extract the result from intent
                destinationPlace = Autocomplete.getPlaceFromIntent(data);
                String destination = destinationPlace.getName()+", "+ destinationPlace.getAddress();
                editText_place_to_go.setText(destination);
                // Make button clickable
                button_calculate.setBackgroundColor(getResources().getColor(R.color.colorPrimary));
                button_calculate.setEnabled(true);


            } else if (resultCode == AutocompleteActivity.RESULT_ERROR) {
                // TODO: Handle the error.
                Status status = Autocomplete.getStatusFromIntent(data);
                Toast.makeText(NewJourneyActivity.this, "Error: " + status.getStatusMessage(), Toast.LENGTH_LONG).show();
                Log.i(TAG, status.getStatusMessage());
            } else if (resultCode == RESULT_CANCELED) {
                // The user canceled the operation.
            }
        }
    }





}
