package jimenezcarlos.android.uoc.edu.camina;

import android.content.Context;
import android.content.Intent;
import android.os.Bundle;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.ProgressBar;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.appcompat.app.AppCompatActivity;
import androidx.appcompat.widget.Toolbar;
import androidx.coordinatorlayout.widget.CoordinatorLayout;
import androidx.recyclerview.widget.RecyclerView;
import androidx.swiperefreshlayout.widget.SwipeRefreshLayout;
import com.google.android.material.snackbar.Snackbar;

import com.google.gson.Gson;
import com.google.gson.JsonObject;
import com.google.gson.reflect.TypeToken;

import java.io.IOException;
import java.lang.reflect.Type;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import jimenezcarlos.android.uoc.edu.menu.LateralMenu;
import jimenezcarlos.android.uoc.edu.rest.adapter.TMBAdapter;
import jimenezcarlos.android.uoc.edu.rest.model.TMBPlannerAPI.Itinerary;
import jimenezcarlos.android.uoc.edu.rest.model.TMBPlannerAPI.Leg;
import jimenezcarlos.android.uoc.edu.rest.model.TMBPlannerAPI.Point;
import jimenezcarlos.android.uoc.edu.rest.model.TMBPlannerAPI.Plan;
import jimenezcarlos.android.uoc.edu.rest.model.TMBPlannerAPI.SerializableOriginDestination;
import jimenezcarlos.android.uoc.edu.util.ItineraryComparator;
import jimenezcarlos.android.uoc.edu.util.LineToTransitIconDrawable;
import jimenezcarlos.android.uoc.edu.util.Utitilies;
import retrofit2.Call;
import retrofit2.Callback;
import retrofit2.Response;

import static java.lang.Boolean.FALSE;
import static java.lang.Boolean.TRUE;

public class RouteSelectionActivity extends AppCompatActivity {

    private static final String TAG = "RouteSelectionActivity";

    // Used to get the transferred data from NewJourneyActivity
    SerializableOriginDestination originDestination;
    private TextView textOrigin;
    private TextView textDestiny;
    // Reference to the progress bar
    private ProgressBar mProgressBar;
    // Reference to the recyclerView
    private RecyclerView recyclerView;
    // This is the adapter to the RecyclerView
    private SimpleItemRecyclerViewAdapter adapter;
    // Reference to the SwipeRefreshLayout
    private SwipeRefreshLayout swipeRefreshLayout;
    // Reference to the CoordinatorLayout, used to show the Snackbar message
    // when no route from origin to destination is available
    private CoordinatorLayout coordinatorLayout;
    // Auxiliary variable to flag whether a swipe to refresh the routes has been done
    boolean swipePerformed = FALSE;

    // To identify the healthiest itinerary in the list
    private final static int HEALTHIEST = 0;
    // To identify the rest of the itineraries in the list
    private final static int OTHER = 1;

    // Auxiliary class to get the name of drawable resources of transit icon lines
    private LineToTransitIconDrawable iconLines;

    private Snackbar snack;
    // This is the object to store the answer of TMBPlannerAPI
    private Plan plan;

    // The argument representing the Object of type SerializableOriginDestination
    public static final String ARG_SERIALIZABLE_ORIGIN_DESTINY= "origin_destiny";
    // The argument representing the Object of type Plan to be passed to MetroStopSelectionActivity
    public static final String ARG_PLAN_OBJECT= "route";
    // The argument which identifies the selected itinerary to be passed to Metro StopSelectionActivity
    public static final String ARG_SELECTED_ITINERARY= "itinerary";

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        // Set the layout for this activity
        setContentView(R.layout.activity_route_selection);
        // Create new object of class Plan.
        plan = new Plan();
        // Get reference to the support action bar in the layout and set it
        Toolbar toolbar = findViewById(R.id.toolbar);
        setSupportActionBar(toolbar);
        // Get references to textViews in the layout
        textOrigin = findViewById(R.id.textOrigin);
        textDestiny = findViewById(R.id.textDestiny);

        //Include the Lateral Menu
        LateralMenu.getDrawer(this,savedInstanceState, toolbar, 2);
        // Get the transferred data from NewJourneyActivity activity.
        Intent intent = getIntent();
        originDestination = (SerializableOriginDestination) intent.getSerializableExtra(NewJourneyActivity.ARG_SERIALIZABLE_ORIGIN_DESTINY);
        // Set textViews with origin and destination addresses
        textOrigin.setText("Origen: " + originDestination.getOriginAddress());
        textDestiny.setText("Destino: " + originDestination.getDestinationAddress());

        //Instantiate a new object of class LineToTransitIconDrawable
        iconLines = new LineToTransitIconDrawable ();

        // Get the reference to the recyclerView in the layout
        recyclerView = findViewById(R.id.item_list);


        // Get the reference to the progress bar
        mProgressBar = findViewById(R.id.routes_progress);
        // Get the reference to the SwipeRefreshLayout and set a listener to call the action
        // that has to be done when swiping
        swipeRefreshLayout = (SwipeRefreshLayout) findViewById(R.id.swipeRefreshLayout);
        swipeRefreshLayout.setColorSchemeColors(getResources().getColor(R.color.colorPrimary), getResources().getColor(R.color.light_primary_color));
        swipeRefreshLayout.setOnRefreshListener(new SwipeRefreshLayout.OnRefreshListener() {
            @Override
            public void onRefresh() {
                // Code to be executed when swipe is performed
                swipePerformed = TRUE;
                //TODO: Before asking again for the available routes, get the updated user's location and update the origin textView and the query before calling TMBPlannerAPI
                AskItinerariesFromOriginToDestination();
            }

        });

        // Get the reference to the CoordinatorLayout
        coordinatorLayout = findViewById(R.id.coordinatorLayout);

        // Call the method which calls TMBPlannerAPI to ask for the routes to get from origin to destination
        // and sets the RecyclerView
        AskItinerariesFromOriginToDestination();

    }

    private void AskItinerariesFromOriginToDestination() {

        // Construct the Map of query parameters to call TMBPlannerAPI to ask for routes to get from origin to destination

        Map<String, String> TMBPlannerQuery = new HashMap<>();
        TMBPlannerQuery.put("showIntermediateStops", "TRUE");
        TMBPlannerQuery.put("mode", "SUBWAY,WALK");
        TMBPlannerQuery.put("fromPlace", originDestination.getOriginCoordinates());
        TMBPlannerQuery.put("toPlace", originDestination.getDestinationCoordinates());

        Call<JsonObject> callTMBPlannerAPIResponse;
        callTMBPlannerAPIResponse = new TMBAdapter().getPlannedRoute(TMBPlannerQuery);
        // If a swipe hasn't been performed (in that case a spinner will be already showed), show progress bar
        if (!swipePerformed) mProgressBar.setVisibility(View.VISIBLE);
        callTMBPlannerAPIResponse.enqueue(new Callback<JsonObject>() {
            @Override
            public void onResponse(@NonNull Call<JsonObject> call, @NonNull Response<JsonObject> response) {

                //If a swipe to refresh was performed, stop the spin
                if (swipePerformed) {
                    swipeRefreshLayout.setRefreshing(false);
                    swipePerformed = FALSE;
                } else mProgressBar.setVisibility(View.GONE);
                JsonObject responseTMBPlannerAPI = response.body();
            if (responseTMBPlannerAPI != null) {

                // If a correct answer is obtained from server, parse the data
                ParseTMBPlannerAPIResponse (responseTMBPlannerAPI);
                // Since Camina has been conceived to always combine walking with Metro
                // Discard all the itineraries which don't include a transit Leg (only walk itineraries)
                DiscardOnlyWalkItineraries ();
                // Finally, if there are itineraries (after discarding only walk ones), setup Recycle View
                if (plan.getItineraries().size()>0)
                    setupRecyclerView();
                else {
                    //Tell the user that there are not itineraries involving public transport
                    snack =  Snackbar.make(coordinatorLayout, getString(R.string.no_transit_itineraries_available), Snackbar.LENGTH_LONG);
                    CoordinatorLayout.LayoutParams params = (CoordinatorLayout.LayoutParams)
                            snack.getView().getLayoutParams();
                    params.setMargins(0,0,0,0);
                    snack.getView().setLayoutParams(params);
                    snack.show();

                }

            } else {

                //No itineraries to go from origin to destiny are available within TMB Metro network
                swipeRefreshLayout.setRefreshing(false);
                mProgressBar.setVisibility(View.GONE);
                //And tell the user
                snack =  Snackbar.make(coordinatorLayout, getString(R.string.no_transit_itineraries_available), Snackbar.LENGTH_LONG);
                CoordinatorLayout.LayoutParams params = (CoordinatorLayout.LayoutParams)
                        snack.getView().getLayoutParams();
                params.setMargins(0,0,0,0);
                snack.getView().setLayoutParams(params);
                snack.show();
                    }
            }

            @Override
            public void onFailure(@NonNull Call<JsonObject> call, @NonNull Throwable t) {
                mProgressBar.setVisibility(View.GONE);
                // TODO: If we get an error from server, inform the user
                   if (t instanceof IOException) {
                       Log.d("RouteSelectionActivity", "Network failure :( inform the user and possibly retry");
                   }
                   else {

                        Log.d("RouteSelectionActivity", "Conversion issue");
                   }
            }
        });

    }

    private void DiscardOnlyWalkItineraries() {

        List<Itinerary> auxItineraries = new ArrayList<>();

        for (Itinerary itinerary: plan.getItineraries()) {
            if (itinerary.getFirstMetroLeg() >=0)
            {
                auxItineraries.add (itinerary);
            }

        plan.setItineraries(auxItineraries);

        }

    }

    private void ParseTMBPlannerAPIResponse(JsonObject responseTMBPlannerAPI) {


        // Convert and parse the date of API call
        plan.setDate(Epoch2DateString(responseTMBPlannerAPI.getAsJsonObject("plan").getAsJsonPrimitive("date").getAsString()));
        // We will use a Gson object to parse some objects into classes automatically
        Gson g = new Gson();
        // Parse JSON objects 'from' and 'to' into Point objects
        Point point = g.fromJson(responseTMBPlannerAPI.getAsJsonObject("plan").getAsJsonObject("from"), Point.class);
        plan.setFrom(point);
        point = g.fromJson(responseTMBPlannerAPI.getAsJsonObject("plan").getAsJsonObject("to"), Point.class);
        plan.setTo(point);

        // Gson class TypeToken is used to assist us in finding the correct Type for our class configuration when trying to get a List of objects of that class
        Type itineraryListType = new TypeToken<ArrayList<Itinerary>>(){}.getType();
        List<Itinerary> itineraries = g.fromJson(responseTMBPlannerAPI.getAsJsonObject("plan").getAsJsonArray("itineraries"), itineraryListType);

            // Order the different itineraries by walktime, walkdistance into descending order,
            // and then total duration into ascending order.
            Collections.sort(itineraries, new ItineraryComparator());
            // Add the sorted list to the plan object
            plan.setItineraries(itineraries);

    }

    public class SimpleItemRecyclerViewAdapter extends RecyclerView.Adapter<SimpleItemRecyclerViewAdapter.ViewHolder> {


        private final RouteSelectionActivity mParentActivity; //The parent activity
        private List<Itinerary> mItineraries; //The items to display in the RecyclerView

        //The constructor
        SimpleItemRecyclerViewAdapter (RouteSelectionActivity parent, List<Itinerary> itineraries) {

            mParentActivity = parent;
            mItineraries = itineraries;

        }


        /**
         * This method creates different RecyclerView.ViewHolder objects based on the item's ordinal
         * The first one is the healthiest, and will be higher and have a different color
         *
         * @param parent   ViewGroup container for the item
         * @param viewType type of view to be inflated
         * @return inflated viewHolder
         */

        @Override
        public ViewHolder onCreateViewHolder(ViewGroup parent, int viewType) {

            View view;
            LayoutInflater inflater = LayoutInflater.from(parent.getContext());
            int layoutResource = 0; // Will store the assigned layout id, depending on the ordinal

            switch (viewType) {
                case HEALTHIEST:
                    layoutResource = R.layout.healthiest_item_list_route;

                    break;

                case OTHER:
                    layoutResource = R.layout.item_list_route;
                    break;
            }

            //The view is inflated with the appropriate layout and then returned
            view = inflater.inflate(layoutResource, parent, false);
            return new ViewHolder(view);

        }


        //Returns the view type of the item at position for the purpose of view recycling.
        @Override
        public int getItemViewType(int position) {

            if (position == 0) return (HEALTHIEST);
            else return (OTHER);
        }


            /**
             * This method internally calls onBindViewHolder(ViewHolder, int) to update the
             * RecyclerView.ViewHolder contents with the item at the given position
             * and also sets up some private fields to be used by RecyclerView.
             *
             * @param holder   The type of RecyclerView.ViewHolder to populate
             * @param position Item position in the viewGroup.
             */

        @Override
        public void onBindViewHolder(final ViewHolder holder, final int position) {

            // The given position determines the itinerary, being the corresponding
            // Itinerary in the same position in the Itineraries list
            // Set holder views with data model values


            holder.mTotalTime.setText(mItineraries.get(position).getDurationFormatted());
            holder.mWalkingDistance.setText(Utitilies.FormatDistanceDoubleToString(mItineraries.get(position).getWalkingDistanceAddingAllLegs()));
            holder.mWalkingTime.setText(Utitilies.FormatTimeIntegerToString(mItineraries.get(position).getWalkingTimeAddingAllLegs()));


            SetTransitIcons (holder, position);

            // A new onClickListener is created and set to the view of ViewHolder
            holder.itemView.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View v) { //When clicked
                    // Obtain context of this Activity
                    final Context context = v.getContext();
                    int currentPos;
                    // Obtain the position of clicked itinerary
                    currentPos = holder.getAdapterPosition();
                    // Start MetroStopSelectionActivity and pass the needed information

                    Intent intent = new Intent(context, MetroStopSelectionActivity.class);
                    intent.putExtra(ARG_SERIALIZABLE_ORIGIN_DESTINY, originDestination);
                    intent.putExtra(ARG_PLAN_OBJECT, plan);
                    intent.putExtra(ARG_SELECTED_ITINERARY, currentPos);
                    context.startActivity(intent);
                    // overridePendingTransition(R.anim.slide_up, R.anim.stay);

                }
            });
        }

        // Return the size of the dataset (invoked by the layout manager)
        @Override
        public int getItemCount() {

            return plan.getItineraries().size();
        }

        class ViewHolder extends RecyclerView.ViewHolder {

            final TextView mWalkingTime;
            final TextView mWalkingDistance;
            final TextView mTotalTime;
            final ImageView [] mTransitIcon = new ImageView[4];

            // Create a constructor that accepts the entire item row
            // and does the view lookups to find each subview
            ViewHolder(View view) {
                super(view);
                //Set the corresponding subviews
                mWalkingTime = view.findViewById (R.id.text_time_walking);
                mWalkingDistance = view.findViewById(R.id.text_distance_walking);
                mTotalTime = view.findViewById(R.id.text_total_time_to_destination);
                for (int i=0; i<4; i++) {
                    String resource = "image_transit_line_icon_"+ String.valueOf(i+1);
                    int id = getResources().getIdentifier(resource, "id", getBaseContext().getPackageName());

                    mTransitIcon[i] = view.findViewById(id);
                }
            }

        }

    }

    private void SetTransitIcons(SimpleItemRecyclerViewAdapter.ViewHolder holder, int position) {

        List<Leg> mLegs = plan.getItineraries().get(position).getLegs();
        int i= 0;

       for (Leg l : mLegs )

       {
           if (l.getMode().equals("SUBWAY")) {

            String resource = iconLines.getResourceName(l.getRoute());
            int id = getResources().getIdentifier(resource, "drawable", getBaseContext().getPackageName());
            holder.mTransitIcon[i].setImageDrawable(getDrawable(id));
            holder.mTransitIcon[i].setVisibility(View.VISIBLE);
            i++;
            resource = "imageView_right_arrow_"+ String.valueOf(i);
            id = getResources().getIdentifier(resource, "id", getBaseContext().getPackageName());
            holder.itemView.findViewById(id).setVisibility(View.VISIBLE);

           }

       }


    }

    private void setupRecyclerView() {

        //We create and set a suitable adapter for the recyclerView
        adapter = new SimpleItemRecyclerViewAdapter(this, plan.getItineraries());
        recyclerView.setAdapter(adapter);

    }


    /**************************/
    /***       UTILS        ***/
    /**************************/

    public static String Epoch2DateString(String epochSeconds) {

        Date updatedDate = new Date(Long.parseLong(epochSeconds));
        SimpleDateFormat sdf = new SimpleDateFormat("dd/MM/yyyy");
        return sdf.format(updatedDate);
    }


}
