package jimenezcarlos.android.uoc.edu.camina;

import android.app.AlertDialog;
import android.app.TaskStackBuilder;
import android.content.DialogInterface;
import android.content.Intent;
import android.graphics.Color;
import android.graphics.PorterDuff;
import android.net.Uri;
import android.os.Bundle;
import android.util.TypedValue;
import android.view.View;
import android.widget.Button;
import android.widget.ImageView;
import android.widget.TextView;
import android.widget.Toast;

import androidx.annotation.NonNull;
import androidx.appcompat.app.AppCompatActivity;
import androidx.appcompat.widget.Toolbar;
import androidx.constraintlayout.widget.ConstraintLayout;
import androidx.constraintlayout.widget.Guideline;

import com.google.android.gms.tasks.OnFailureListener;
import com.google.android.gms.tasks.OnSuccessListener;
import com.google.firebase.auth.FirebaseAuth;
import com.google.firebase.auth.FirebaseUser;
import com.google.firebase.firestore.FirebaseFirestore;

import java.util.Calendar;

import jimenezcarlos.android.uoc.edu.menu.LateralMenu;
import jimenezcarlos.android.uoc.edu.rest.model.Path;
import jimenezcarlos.android.uoc.edu.util.LineToTransitIconDrawable;

import static jimenezcarlos.android.uoc.edu.util.Utitilies.FormatDistanceDoubleToString;
import static jimenezcarlos.android.uoc.edu.util.Utitilies.FormatTimeIntegerToString;

public class StartItineraryActivity extends AppCompatActivity implements View.OnClickListener {


    private static final String TAG = "StartItineraryActivity";

    // To save references to layout items
    private ImageView[] mTransitIcon = new ImageView[4];
    private ImageView[] mStationPoints = new ImageView[8];
    private ImageView[] mMetroLines = new ImageView[4];
    private TextView[] mStationNames = new TextView[5];
    private TextView[] mStationsInLine = new TextView[4];
    private Guideline[] mVerticalGuidelines = new Guideline[6];

    private TextView mStartWalkingDistance;
    private TextView mStartWalkingTime;
    private TextView mEndWalkingDistance;
    private TextView mEndWalkingTime;

    private TextView mSkippedStations;
    private TextView mWalkTime;
    private TextView mWalkDistance;
    private TextView mCalories;
    private TextView mOMSActivity;
    private Button mStartButton;
    // To get information from MetroStopSelectionActivity, and save last route to FireStore
    private Path finalItinerary;
    // Flag variable that indicates whether the user skipped all Metro stops in the first Metro line
    private boolean allStopsInFirstLineSkipped=false;

    //FirestoreDB instance
    private FirebaseFirestore firestoreDB;
    FirebaseUser user;

    // Auxiliary class to get the name of drawable resources of transit icon lines
    private LineToTransitIconDrawable iconLines;


    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        // Set the layout for this activity
        setContentView(R.layout.activity_start_itinerary);

        // Get reference to the support action bar in the layout and set it
        Toolbar toolbar = findViewById(R.id.toolbar);
        setSupportActionBar(toolbar);

        //Include the Lateral Menu
        LateralMenu.getDrawer(this,savedInstanceState, toolbar, 2);
        // Get the transferred data from NewJourneyActivity activity.
        Intent intent = getIntent();
        finalItinerary = (Path) intent.getSerializableExtra(MetroStopSelectionActivity.ARG_FINAL_PATH);
        allStopsInFirstLineSkipped = intent.getBooleanExtra(MetroStopSelectionActivity.ARG_SKIPPED_ALL_STOPS_IN_FIRST_METRO_LINE, false);

        //Instantiate a new object of class LineToTransitIconDrawable
        iconLines = new LineToTransitIconDrawable ();

        // Get an instance of Firestore
        firestoreDB = FirebaseFirestore.getInstance();
        //Get user
        user = FirebaseAuth.getInstance().getCurrentUser();
        setInformationInLayout();

    }



    void setInformationInLayout () {

        // Get references to every View in the layout and set the corresponding information

        mSkippedStations = findViewById(R.id.textView_si_skipped);
        if (finalItinerary.getSkippedStations()==0)
            mSkippedStations.setText(getString(R.string.introductory_text_0));

        else if (allStopsInFirstLineSkipped) {
            mSkippedStations.setText (getString (R.string.introductory_text_all, finalItinerary.getSkippedStations()));
        }

        else if (finalItinerary.getSkippedStations()==1)
            mSkippedStations.setText(getString(R.string.introductory_text_1));

        else mSkippedStations.setText(getString(R.string.introductory_text_multiple, finalItinerary.getSkippedStations()));

        mWalkTime = findViewById(R.id.textView_quantitative_feature_1);
        mWalkTime.setText (getString(R.string.walk_time, FormatTimeIntegerToString(finalItinerary.getWalkTime())));
        mWalkDistance = findViewById(R.id.textView_quantitative_feature_2);
        mWalkDistance.setText(getString(R.string.walk_distance, FormatDistanceDoubleToString(finalItinerary.getWalkDistance())));
        mCalories = findViewById(R.id.textView_quantitative_feature_3);
        mCalories.setText(getString(R.string.calories, finalItinerary.getCalories()));
        mOMSActivity = findViewById(R.id.textView_quantitative_feature_4);
        mOMSActivity.setText(getString(R.string.OMS, finalItinerary.getOMSActivityRatio()));

        mStartWalkingTime = findViewById(R.id.textView_WalkingTime_1);
        mStartWalkingDistance = findViewById(R.id.textView_WalkingDistance_1);
        mEndWalkingTime = findViewById(R.id.textView_WalkingTime_2);
        mEndWalkingDistance = findViewById(R.id.textView_WalkingDistance_2);

        mStartWalkingTime.setText(FormatTimeIntegerToString(finalItinerary.getStartWalkingTime()));
        mStartWalkingDistance.setText(FormatDistanceDoubleToString(finalItinerary.getStartWalkingDistance()));
        mEndWalkingTime.setText(FormatTimeIntegerToString(finalItinerary.getEndWalkingTime()));
        mEndWalkingDistance.setText(FormatDistanceDoubleToString(finalItinerary.getEndWalkingDistance()));
        mStartButton = findViewById(R.id.buttonStart);
        mStartButton.setOnClickListener(this);

        //Get the reference to Metro station names
        for (int i=0; i<=4; i++) {
            String resource = "textView_MS_" + String.valueOf(i + 1);
            int id = getResources().getIdentifier(resource, "id", getBaseContext().getPackageName());
            mStationNames[i] = findViewById(id);

        }


        // Get references to every View in the layout
        for (int i=0; i<=7; i++) {
            String resource = "icon_metro_line_start_stop_" + String.valueOf(i + 1);
            int id = getResources().getIdentifier(resource, "id", getBaseContext().getPackageName());
            mStationPoints[i] = findViewById(id);

        }

        for (int i=0; i<=3; i++) {
            String resource = "image_transit_line_icon_" + String.valueOf(i + 1);
            int id = getResources().getIdentifier(resource, "id", getBaseContext().getPackageName());
            mTransitIcon[i] = findViewById(id);
            resource = "imageView_MetroLine_"+ String.valueOf(i + 1);
            id = getResources().getIdentifier(resource, "id", getBaseContext().getPackageName());
            mMetroLines[i] = findViewById(id);
            resource = "textView_stops_line_"+ String.valueOf(i + 1);
            id = getResources().getIdentifier(resource, "id", getBaseContext().getPackageName());
            mStationsInLine[i] = findViewById(id);

        }

        // Get references to every guideline in the layout
        for (int i=0; i<=5; i++) {
            String resource = "guideline_vertical_" + String.valueOf(i + 1);
            int id = getResources().getIdentifier(resource, "id", getBaseContext().getPackageName());
            mVerticalGuidelines[i] = findViewById(id);
        }


        // Get the number of different metro lines, and adjust the layout (maximum is 4)

        int metroLines = finalItinerary.getMetroLines().size();
        for (int i=0; i<metroLines; i++)
        {
            String resource = iconLines.getResourceName(finalItinerary.getMetroLines().get(i));
            int id = getResources().getIdentifier(resource, "drawable", getBaseContext().getPackageName());
            mTransitIcon[i].setImageDrawable(getDrawable(id));
            mTransitIcon[i].setVisibility(View.VISIBLE);
            // Get this Metro line color, and use it to tint the corresponding Metro line and points in Layout
            int metroLineColor = Color.parseColor("#"+finalItinerary.getMetroLineColors().get(i));
            mMetroLines[i].setColorFilter(metroLineColor, PorterDuff.Mode.MULTIPLY);
            mMetroLines[i].setVisibility(View.VISIBLE);
            mStationPoints[i*2].setColorFilter(metroLineColor, PorterDuff.Mode.MULTIPLY);
            mStationPoints[i*2+1].setColorFilter(metroLineColor, PorterDuff.Mode.MULTIPLY);
            mStationPoints[i*2].setVisibility(View.VISIBLE);
            mStationPoints[i*2+1].setVisibility(View.VISIBLE);
        }

        if (allStopsInFirstLineSkipped) {

            mStationNames[0].setText(finalItinerary.getStationNames().get(0).getStationNamesInLine().get(0));
            mStationsInLine[0].setText(getNumberOfStationsString(finalItinerary.getStationNames().get(0).getStationNamesInLine().size()));
        }

        else {

            mStationNames[0].setText(finalItinerary.getStationNames().get(0).getStationNamesInLine().get(finalItinerary.getSkippedStations()));
            mStationsInLine[0].setText(getNumberOfStationsString(finalItinerary.getStationNames().get(0).getStationNamesInLine().size() - finalItinerary.getSkippedStations() - 1));

        }
        mStationNames[0].setVisibility(View.VISIBLE);
        mStationsInLine[0].setVisibility(View.VISIBLE);


        for (int i=1; i<metroLines; i++)

        {
            mStationNames[i].setText(finalItinerary.getStationNames().get(i-1).getStationNamesInLine().get(finalItinerary.getStationNames().get(i-1).getStationNamesInLine().size()-1));
            mStationNames[i].setVisibility(View.VISIBLE);
            mStationsInLine[i].setText(getNumberOfStationsString (finalItinerary.getStationNames().get(i).getStationNamesInLine().size()-1));
            mStationsInLine[i].setVisibility(View.VISIBLE);

        }

        mStationNames[metroLines].setText(finalItinerary.getStationNames().get(metroLines-1).getStationNamesInLine().get(finalItinerary.getStationNames().get(metroLines-1).getStationNamesInLine().size()-1));
        mStationNames[metroLines].setVisibility(View.VISIBLE);

         // Rearrange Layout: move the 2 last vertical guidelines to the appropriate position
        // According to the number of routes made in Metro
        ConstraintLayout.LayoutParams params = (ConstraintLayout.LayoutParams) mVerticalGuidelines[4].getLayoutParams();
        params.guideBegin = (int) TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_DIP, 90+metroLines*100, getResources().getDisplayMetrics()); // Convert from dp's to pixels
        mVerticalGuidelines[4].setLayoutParams(params);
        params = (ConstraintLayout.LayoutParams) mVerticalGuidelines[5].getLayoutParams();
        params.guideBegin = (int) TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_DIP, 90+(metroLines+1)*100, getResources().getDisplayMetrics()); // Convert from dp's to pixels
        mVerticalGuidelines[5].setLayoutParams(params);

    }

    @Override
    public void onClick(View view){

        if (view.getId()==R.id.buttonStart) { //If buttonStart was clicked

            //
            AlertDialog.Builder alertDialog = new AlertDialog.Builder(StartItineraryActivity.this);
            alertDialog.setTitle("Iniciar itinerario");
            if (allStopsInFirstLineSkipped)
                alertDialog.setMessage(getString(R.string.alert_before_start_itinerary, finalItinerary.getStationNames().get(0).getStationNamesInLine().get(0)));
            else alertDialog.setMessage(getString(R.string.alert_before_start_itinerary, finalItinerary.getStationNames().get(0).getStationNamesInLine().get(finalItinerary.getSkippedStations())));
            alertDialog.setPositiveButton ("IR",
                    new DialogInterface.OnClickListener() {
                        public void onClick(DialogInterface dialog, int which) {
                            dialog.dismiss();

                            if (user!=null) { //If user exists, update field LastRoute with finalItinerary value
                                //Update the date before uploading to server
                                finalItinerary.setDate(Calendar.getInstance().getTime());
                                firestoreDB.collection("users").document(user.getUid())//We use the user.getUid as document ID
                                        .update("lastRoute", finalItinerary)
                                        .addOnSuccessListener(new OnSuccessListener<Void>()  {
                                            @Override
                                            public void onSuccess(Void aVoid) {
                                                Toast.makeText(StartItineraryActivity.this,
                                                        getString(R.string.last_itinerary_successfully_saved),
                                                        Toast.LENGTH_SHORT).show();

                                                // Create new intent to start Google Maps
                                                Uri gmmIntentUri = Uri.parse("google.navigation:q="+finalItinerary.getLatFirstMetroStop()+","+finalItinerary.getLonFirstMetroStop()+"&mode=w");
                                                Intent mapIntent = new Intent(Intent.ACTION_VIEW, gmmIntentUri);
                                                mapIntent.setPackage("com.google.android.apps.maps");

                                                //Create new intent to start MapsActivity, clearing the task stack
                                                Intent mapsActivityIntent = new Intent(getApplicationContext(), MapsActivity.class);
                                                mapsActivityIntent.addFlags(Intent.FLAG_ACTIVITY_CLEAR_TASK|Intent.FLAG_ACTIVITY_NEW_TASK);
                                                // Create new intent to start CertifyItineraryActivity
                                                Intent certifyIntent = new Intent(getApplicationContext(), CertifyItineraryActivity.class);
                                                // Create a new stack, being Google Maps on top of CertifyItineraryActivity, and start activities
                                                TaskStackBuilder.create(getApplicationContext())
                                                        .addNextIntent(mapsActivityIntent)
                                                        .addNextIntentWithParentStack( certifyIntent )
                                                        .addNextIntentWithParentStack( mapIntent )
                                                        .startActivities();
                                                //And finish this activity
                                                finish();
                                            }
                                        })
                                        .addOnFailureListener(e -> Toast.makeText(StartItineraryActivity.this,
                                                getString(R.string.last_itinerary_saving_error),
                                                Toast.LENGTH_SHORT).show());

                            }


                        }
                    });

            alertDialog.setNegativeButton("CANCELAR", new DialogInterface.OnClickListener() {
                public void onClick(DialogInterface dialog, int id) {
                    dialog.dismiss();
                }
            });

            alertDialog.show();


        }

    }

    // This auxiliary method manages singular/plural number of stations message
    public String getNumberOfStationsString (int nOfStations) {

        if (nOfStations==1)
            return getString(R.string.one_station);
        else return (getString(R.string.number_stations, nOfStations));

    }

}
