package jimenezcarlos.android.uoc.edu.camina;

import android.content.Context;
import android.net.Uri;
import android.os.Bundle;

import androidx.constraintlayout.widget.ConstraintLayout;
import androidx.fragment.app.Fragment;

import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ProgressBar;
import android.widget.TextView;
import android.widget.Toast;

import com.google.firebase.auth.FirebaseAuth;
import com.google.firebase.auth.FirebaseUser;
import com.google.firebase.firestore.CollectionReference;
import com.google.firebase.firestore.DocumentReference;
import com.google.firebase.firestore.DocumentSnapshot;
import com.google.firebase.firestore.FirebaseFirestore;

import java.util.Calendar;

import jimenezcarlos.android.uoc.edu.rest.model.CaminaUser;
import jimenezcarlos.android.uoc.edu.rest.model.DailyStatistics;
import jimenezcarlos.android.uoc.edu.rest.model.Registry;
import jimenezcarlos.android.uoc.edu.util.Utitilies;


public class StatisticsTotalFragment extends Fragment {
    // References to views in the layout
    private TextView mSkippedStations;
    private TextView mCalories;
    private TextView mDistance;
    private TextView mTime;
    private ProgressBar mProgressBar;
    private ConstraintLayout mContent;

    private CaminaUser mCaminaUser;

    private OnFragmentInteractionListener mListener;

    public StatisticsTotalFragment() {
        // Required empty public constructor
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
    }

    private void readStatistics() {

        //Hide content and show the progress bar
        showLoading();
        // Get an instance of FireStore
        FirebaseFirestore fireStoreDB = FirebaseFirestore.getInstance();
        //Get user
        FirebaseUser user = FirebaseAuth.getInstance().getCurrentUser();
        //TODO: if user==null, tell the user he must be logged-in and send him to sign-in
        //Get a reference to the user's document
        DocumentReference userDocumentReference = fireStoreDB.collection("users").document(user.getUid());
        //Execute query to get user's global stats
        userDocumentReference
                .get()
                .addOnCompleteListener(task -> {
                    if (task.isSuccessful()) {
                        DocumentSnapshot document = task.getResult();
                        if (document.exists()) {
                            //Get the dailyStatistics from server
                            mCaminaUser = task.getResult().toObject(CaminaUser.class);
                            showStatistics(mCaminaUser);

                        } else {

                            mProgressBar.setVisibility(View.INVISIBLE);
                            Toast.makeText(this.getActivity(),
                                    R.string.show_statistics_error,
                                    Toast.LENGTH_LONG).show();

                        }

                    } else {

                        mProgressBar.setVisibility(View.INVISIBLE);
                        Toast.makeText(this.getActivity(),
                                R.string.show_statistics_error,
                                Toast.LENGTH_LONG).show();

                    }



                });



    }


    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        // Inflate the layout for this fragment
        View rootView =  inflater.inflate(R.layout.fragment_st_day, container, false);

        // Get references to layout elements
        mSkippedStations = rootView.findViewById(R.id.st_textView_metroStops);
        mCalories = rootView.findViewById(R.id.st_textView_calories);
        mDistance = rootView.findViewById(R.id.st_textView_distance);
        mTime = rootView.findViewById(R.id.st_textView_time);

        mProgressBar = rootView.findViewById(R.id.st_progress);
        mContent = rootView.findViewById(R.id.st_content);
        readStatistics();
        return rootView;
    }



    @Override
    public void onAttach(Context context) {
        super.onAttach(context);
        if (context instanceof OnFragmentInteractionListener) {
            mListener = (OnFragmentInteractionListener) context;
        } else {
            throw new RuntimeException(context.toString()
                    + " must implement OnFragmentInteractionListener");
        }
    }

    @Override
    public void onDetach() {
        super.onDetach();
        mListener = null;
    }

    /**
     * This interface must be implemented by activities that contain this
     * fragment to allow an interaction in this fragment to be communicated
     * to the activity and potentially other fragments contained in that
     * activity.
     * <p>
     * See the Android Training lesson <a href=
     * "http://developer.android.com/training/basics/fragments/communicating.html"
     * >Communicating with Other Fragments</a> for more information.
     */
    public interface OnFragmentInteractionListener {
        // TODO: Update argument type and name
        void onFragmentInteraction(Uri uri);
    }

    private void showLoading() {
        //Show progressbar and hide content
        mProgressBar.setVisibility(View.VISIBLE);
        mContent.setVisibility(View.GONE);
    }

    private void showStatistics(CaminaUser caminaUser) {

        mSkippedStations.setText(getString(R.string.st_metro_stops,caminaUser.getTotalSkippedStations()));
        mCalories.setText (getString(R.string.st_calories, caminaUser.getTotalCalories()));
        mDistance.setText (Utitilies.FormatDistanceDoubleToString((double)caminaUser.getTotalDistance()));
        mTime.setText(Utitilies.FormatTimeIntegerToString(caminaUser.getTotalTime()));

        //Show content and hid progressbar
        mProgressBar.setVisibility(View.GONE);
        mContent.setVisibility(View.VISIBLE);
    }

}
