package jimenezcarlos.android.uoc.edu.camina;

import android.app.AlertDialog;
import android.app.TaskStackBuilder;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.os.Bundle;
import android.view.View;
import android.widget.Button;
import android.widget.Toast;

import androidx.annotation.NonNull;
import androidx.appcompat.app.AppCompatActivity;

import com.firebase.ui.auth.AuthUI;
import com.firebase.ui.auth.IdpResponse;
import com.google.android.gms.tasks.OnCompleteListener;
import com.google.android.gms.tasks.Task;
import com.google.firebase.auth.FirebaseAuth;
import com.google.firebase.auth.FirebaseUser;
import com.google.firebase.firestore.DocumentSnapshot;
import com.google.firebase.firestore.FirebaseFirestore;

import java.util.Arrays;
import java.util.Collections;
import java.util.List;

import jimenezcarlos.android.uoc.edu.rest.model.CaminaUser;

import static java.lang.Boolean.TRUE;


public class UserIdentificationActivity extends AppCompatActivity {

    private static final int RC_SIGN_IN = 123;
    private Button button_start_walking;
    private boolean userWithoutCaminaUser = false;

    // The argument representing the Object of type SerializableOriginDestination
    public static String ARG_NEW_PROFILE = "new profile";

    protected void onCreate(Bundle savedInstanceState) {

        super.onCreate(savedInstanceState);

        // Set the layout
        setContentView(R.layout.activity_user_identification);


        // Check if there is already a user
        checkCurrentUser();

    }

    public void checkCurrentUser() {

        FirebaseUser user = FirebaseAuth.getInstance().getCurrentUser();
        if (user != null) {
            //Security check.
            //In case the user has been deleted and the auth token is still valid
            //Or the user is valid but it has not been stored in FireStore BD yet, due to a previous error when creating the profile
            //Or the user closed the App before the profile creation
            FirebaseFirestore firestoreDB =  FirebaseFirestore.getInstance();
            firestoreDB.collection("users")
                    .document(user.getUid())
                    .get()
                    .addOnCompleteListener(new OnCompleteListener<DocumentSnapshot>() {
                        @Override
                        public void onComplete(@NonNull Task<DocumentSnapshot> task) {
                            if (task.isSuccessful()) {
                                DocumentSnapshot document = task.getResult();
                                if (!document.exists()) {
                                    //The user does not really exist in FireBaseAuth, or it does exist but did'nt complete the profile creation successfully
                                    //So we force sign out
                                    AuthUI.getInstance()
                                            .signOut(UserIdentificationActivity.this)
                                            .addOnCompleteListener(new OnCompleteListener<Void>() {
                                                public void onComplete(@NonNull Task<Void> task) {
                                                    userWithoutCaminaUser = true;
                                                    createSignInIntent();
                                                }
                                            });


                                } else {

                                    // User really exists, has a profile, and is signed in

                                    if  (document.toObject(CaminaUser.class).getLastRoute()!=null)

                                    { //The user has an previous itinerary which has not been certified yet,
                                      //so send him to CertificationActivity

                                        AlertDialog.Builder alertDialog = new AlertDialog.Builder(UserIdentificationActivity.this);
                                        alertDialog.setTitle(R.string.existing_itinerary_to_certify_title);
                                        alertDialog.setMessage(R.string.existing_itinerary_to_certify_message);
                                        alertDialog.setPositiveButton("SÍ",
                                                (dialog, which) -> {
                                                    dialog.dismiss();
                                                    //Create new intent to start MapsActivity
                                                    Intent mapsActivityIntent = new Intent(getApplicationContext(), MapsActivity.class);
                                                    // Create new intent to start CertifyItineraryActivity
                                                    Intent certifyIntent = new Intent(getApplicationContext(), CertifyItineraryActivity.class);
                                                    // Create a new stack, being CertifyItineraryActivity on top of MapsActivity, and start activities
                                                    TaskStackBuilder.create(getApplicationContext())
                                                            .addNextIntent(mapsActivityIntent)
                                                            .addNextIntentWithParentStack( certifyIntent )
                                                            .startActivities();
                                                    //And finish this activity
                                                    finish();
                                                });

                                        alertDialog.setNegativeButton("NO",
                                                (dialog, id) -> {
                                                    dialog.dismiss();
                                                    Intent intent = new Intent(UserIdentificationActivity.this, MapsActivity.class);
                                                    startActivity(intent);
                                                    finish();
                                                });

                                        alertDialog.show();




                                    }
                                    else //Send the user to the MapsActivity
                                    {
                                        Intent intent = new Intent(UserIdentificationActivity.this, MapsActivity.class);
                                        startActivity(intent);
                                        finish();

                                    }
                                }

                            } else {
                                Toast.makeText(UserIdentificationActivity.this,
                                        "Error recuperando los datos de tu perfil",
                                        Toast.LENGTH_SHORT).show();
                            }
                        }

                    });


        } else {
            // No user is signed in
            // So we call the method to identify the user
            createSignInIntent();
        }

    }



    public void createSignInIntent() {


        List<AuthUI.IdpConfig> providers = Arrays.asList(
                new AuthUI.IdpConfig.EmailBuilder().build(),
                new AuthUI.IdpConfig.GoogleBuilder().build(),
                new AuthUI.IdpConfig.FacebookBuilder().build());

        // Create and launch sign-in intent
        startActivityForResult(
                AuthUI.getInstance()
                        .createSignInIntentBuilder()
                        .setLogo(R.drawable.logo_camina)      // Set logo drawable
                        .setTheme(R.style.AppTheme)      // Set theme
                        .setAvailableProviders(providers)
                        .setIsSmartLockEnabled(false)
                        .build(),
                RC_SIGN_IN);

    }


    // [START auth_fui_result]
    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
        super.onActivityResult(requestCode, resultCode, data);

        if (requestCode == RC_SIGN_IN) {

            IdpResponse response = IdpResponse.fromResultIntent(data);

            if (resultCode == RESULT_OK) {
                // Successfully signed in
                // Check whether it was an existing user or a new account has been created

                if (response.isNewUser() || userWithoutCaminaUser) {
                    // If it's a new user, welcome him and proceed to UserProfileActivity
                    // Set the app's theme, and thus remove the one used in the splash screen
                    setTheme(R.style.AppTheme);
                    setContentView(R.layout.new_account_created);
                    button_start_walking = findViewById(R.id.nac_button_startWalking);

                    button_start_walking.setOnClickListener(new View.OnClickListener() {
                        public void onClick(View v) {
                            // When button Start Walking is clicked
                            // a new intent to start UserProfileActivity is created
                            final Context context = v.getContext();
                            Intent intent = new Intent(context, UserProfileActivity.class);
                            intent.putExtra(ARG_NEW_PROFILE, TRUE);
                            startActivity(intent);
                            finish();
                        }

                    });
                }

                else {
                    //It was an existing user, so proceed to MapsActivity
                    Intent intent = new Intent(this, MapsActivity.class);
                    startActivity(intent);
                    finish();

                }

                // ...
            } else {
                // Sign in failed. If response is null the user canceled the
                // sign-in flow using the back button. Otherwise check
                // response.getError().getErrorCode() and handle the error.
                // ...
                if (response == null) {
                    // back button is pressed


                }

            }
        }
    }
    // [END auth_fui_result]





    public void signOut() {

        AuthUI.getInstance()
                .signOut(this)
                .addOnCompleteListener(new OnCompleteListener<Void>() {
                    public void onComplete(@NonNull Task<Void> task) {
                        // ...
                    }
                });

    }

    public void delete() {

        AuthUI.getInstance()
                .delete(this)
                .addOnCompleteListener(new OnCompleteListener<Void>() {
                    @Override
                    public void onComplete(@NonNull Task<Void> task) {
                        // ...
                    }
                });

    }

    public void onRestart () {
        super.onRestart();
        // If there is not user logged in
        // Inform the user that must be logged-in to use Camina, and call createSignInIntent

        if (FirebaseAuth.getInstance().getCurrentUser()==null) {

            AlertDialog alertDialog = new AlertDialog.Builder(this).create();
            alertDialog.setTitle(this.getString(R.string.log_in_required_title));
            alertDialog.setMessage(this.getString(R.string.log_in_required_message));
            alertDialog.setButton(AlertDialog.BUTTON_NEUTRAL, "OK",
                    new DialogInterface.OnClickListener() {
                        public void onClick(DialogInterface dialog, int which) {
                            dialog.dismiss();
                            createSignInIntent();
                        }
                    });

            alertDialog.show();
        }

    }


}


