package jimenezcarlos.android.uoc.edu.camina;

import android.Manifest;
import android.app.Activity;
import android.app.AlertDialog;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.graphics.Bitmap;
import android.graphics.drawable.BitmapDrawable;
import android.net.Uri;
import android.os.Bundle;
import android.view.View;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ProgressBar;
import android.widget.Spinner;
import android.widget.TextView;
import android.widget.Toast;

import androidx.annotation.NonNull;
import androidx.annotation.StringRes;
import androidx.appcompat.app.AppCompatActivity;
import androidx.appcompat.widget.Toolbar;
import androidx.core.app.ActivityCompat;
import androidx.core.content.ContextCompat;

import com.firebase.ui.auth.AuthUI;
import com.google.android.gms.tasks.Continuation;
import com.google.android.gms.tasks.OnCompleteListener;
import com.google.android.gms.tasks.OnFailureListener;
import com.google.android.gms.tasks.OnSuccessListener;
import com.google.android.gms.tasks.Task;
import com.google.firebase.auth.FirebaseAuth;
import com.google.firebase.auth.FirebaseUser;
import com.google.firebase.auth.UserProfileChangeRequest;
import com.google.firebase.firestore.DocumentReference;
import com.google.firebase.firestore.DocumentSnapshot;
import com.google.firebase.firestore.FirebaseFirestore;
import com.google.firebase.storage.FirebaseStorage;
import com.google.firebase.storage.StorageReference;
import com.google.firebase.storage.UploadTask;
import com.squareup.picasso.Picasso;

import java.io.ByteArrayOutputStream;
import java.util.HashMap;
import java.util.Map;

import de.hdodenhof.circleimageview.CircleImageView;
import jimenezcarlos.android.uoc.edu.menu.LateralMenu;
import jimenezcarlos.android.uoc.edu.rest.model.CaminaUser;
import jimenezcarlos.android.uoc.edu.util.Utitilies;

import static java.lang.Boolean.FALSE;

public class UserProfileActivity extends AppCompatActivity implements View.OnClickListener, View.OnFocusChangeListener {


    // To store the references to the views
    private EditText completeName;
    private TextView email;
    private CircleImageView profileImage;
    private TextView age;
    private Spinner spinner_GenreSelection;
    private TextView height;
    private TextView weight;
    private TextView city;
    private TextView country;
    private Button mButton;
    private View content;
    private ProgressBar progressBar;

    // Flag variables
    private boolean newProfile;
    private boolean pictureChanged;
    private boolean userWithoutCaminaUser = false;

    // Request code
    private final int REQUEST_CODE_GALLERY = 700;
    private final int REQUEST_PERMISSION_STORAGE_READ = 701;

    //Other variables
    Uri selectedImage;
    //FirestoreDB instance
    private FirebaseFirestore firestoreDB;
    FirebaseUser user;


    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_profile);
        Toolbar toolbar = findViewById(R.id.toolbar);
        setSupportActionBar(toolbar);

        firestoreDB = FirebaseFirestore.getInstance();

        // If we get to this Activity from UserIdentificationActivity, get the intent and de value passed
        // to know whether there is already a complete profile or no

        Intent intent = getIntent();
        newProfile = intent.getBooleanExtra(UserIdentificationActivity.ARG_NEW_PROFILE, FALSE);
        pictureChanged = FALSE; //Used to know whether the user has added a picture when identification has been done with email

        initViews();

        // Set listeners
        profileImage.setOnClickListener(this);
        profileImage.setOnFocusChangeListener(this);
        mButton.setOnClickListener(this);

        showLoading();

        //Get user
        user = FirebaseAuth.getInstance().getCurrentUser();
        if (user != null) {

            if (user.getDisplayName() != null) {
                // Set the user name in the corresponding EditText
                completeName.setText(user.getDisplayName());

            }
            if (user.getEmail() != null) {
                // Set the email in the corresponding TextView
                email.setText(user.getEmail());

            }

            if (user.getPhotoUrl() != null) {
                // Set the picture in the corresponding ImageView

                Picasso.get()
                        .load(user.getPhotoUrl())
                        .placeholder(R.drawable.man)
                        .error(R.drawable.man)
                        .into(profileImage);
            }
        }

        if (newProfile) {
            mButton.setText(R.string.create_profile);
            showProfileContent();
            setFieldsEditable();
        }
        if (!newProfile) { // The lateral menu is only available if it's not a new profile
            //Security check (in case the user has been deleted and the auth token is still valid)´
            /*
            firestoreDB.collection("users")
                    .document(user.getUid())
                    .get()
                    .addOnCompleteListener(new OnCompleteListener<DocumentSnapshot>() {
                        @Override
                        public void onComplete(@NonNull Task<DocumentSnapshot> task) {
                            if (task.isSuccessful()) {
                                DocumentSnapshot document = task.getResult();
                                if (!document.exists()) { //If there was a document with that id, user.getUid()
                                  //The user does not really exist.
                                  //So we force sign out

                                    AuthUI.getInstance()
                                            .signOut(UserProfileActivity.this)
                                            .addOnCompleteListener(new OnCompleteListener<Void>() {
                                                public void onComplete(@NonNull Task<Void> task) {
                                                    //Inform the user that sign out has been performed and go to UserIdentificationActivity
                                                    AlertDialog alertDialog = new AlertDialog.Builder(UserProfileActivity.this).create();
                                                    alertDialog.setTitle(UserProfileActivity.this.getString(R.string.sign_out_title));
                                                    alertDialog.setMessage(UserProfileActivity.this.getString(R.string.sign_out_message));
                                                    alertDialog.setButton(AlertDialog.BUTTON_NEUTRAL, "OK",
                                                            new DialogInterface.OnClickListener() {
                                                                public void onClick(DialogInterface dialog, int which) {
                                                                    dialog.dismiss();
                                                                    Intent intent = new Intent(UserProfileActivity.this, UserIdentificationActivity.class);
                                                                    startActivity(intent);
                                                                    finish();
                                                                }
                                                            });

                                                    alertDialog.show();
                                                }
                                            });


                                }
                            } else {
                                Toast.makeText(UserProfileActivity.this,
                                        "Error recuperando los datos de tu perfil",
                                        Toast.LENGTH_SHORT).show();
                            }
                        }

                    });


             */
            //Include the Lateral Menu
            LateralMenu.getDrawer(this,savedInstanceState, toolbar, 8);
            setFieldsNotEditable();
            readProfile();

        }


        spinner_GenreSelection = (Spinner) findViewById(R.id.ap_spinner_Genre);
        // Create an ArrayAdapter using the string array and a default spinner layout
        ArrayAdapter<CharSequence> adapterSpinner = ArrayAdapter.createFromResource(this, R.array.genre_array, android.R.layout.simple_spinner_item);
        // Specify the layout to use when the list of choices appears
        adapterSpinner.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
        // Apply the adapter to the spinner
        spinner_GenreSelection.setAdapter(adapterSpinner);

    }

    private void readProfile() {

        firestoreDB.collection("users")
                .document(user.getUid())
                .get()
                .addOnCompleteListener(new OnCompleteListener<DocumentSnapshot>() {
                    @Override
                    public void onComplete(@NonNull Task<DocumentSnapshot> task) {
                        if (task.isSuccessful()) {
                            DocumentSnapshot document = task.getResult();
                            if (document.exists()) { //If there was a document with that id, user.getUid()
                                showProfileContent();
                                CaminaUser c = task.getResult().toObject(CaminaUser.class);
                                completeName.setText(c.getCompleteName());
                                age.setText(String.valueOf(c.getAge()));
                                //Set the appropriate value in the spinner
                                String compareValue = c.getGenre();
                                ArrayAdapter<CharSequence> adapter = ArrayAdapter.createFromResource(UserProfileActivity.this, R.array.genre_array, android.R.layout.simple_spinner_item);
                                adapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
                                spinner_GenreSelection.setAdapter(adapter);
                                if (compareValue != null) {
                                    int spinnerPosition = adapter.getPosition(compareValue);
                                    spinner_GenreSelection.setSelection(spinnerPosition);
                                }
                                height.setText(String.valueOf(c.getHeight()));
                                weight.setText(String.valueOf(c.getWeight()));
                                city.setText(c.getCity());
                                country.setText(c.getCountry());
                            }
                        } else {
                            Toast.makeText(UserProfileActivity.this,
                                    "Error recuperando los datos de tu perfil",
                                    Toast.LENGTH_SHORT).show();
                        }
                    }

                });

    }


    private void pickFromGallery(){
        //Create an Intent with action as ACTION_PICK
        Intent intent=new Intent(Intent.ACTION_PICK);
        // Sets the type as image, to ensure only components of type image are selected
        intent.setType("image/*");
        //We pass an extra array with the accepted mime types. This will ensure only components with these MIME types as targeted.
        String[] mimeTypes = {"image/jpeg", "image/png"};
        intent.putExtra(Intent.EXTRA_MIME_TYPES,mimeTypes);
        // Launching the Intent
        startActivityForResult(intent,REQUEST_CODE_GALLERY);
    }

    @Override
    public void onActivityResult(int requestCode,int resultCode,Intent data){
        super.onActivityResult(requestCode, resultCode, data);
        // Result code is RESULT_OK only if the user selects an Image
        if (resultCode == Activity.RESULT_OK)
            switch (requestCode){
                case REQUEST_CODE_GALLERY:
                    //data.getData return the content URI for the selected Image
                    selectedImage = data.getData();
                    profileImage.setImageURI(selectedImage);
                    pictureChanged = true;


            }
    }



    private void showMessage(@StringRes int stringId) {
        Toast.makeText(this, stringId, Toast.LENGTH_LONG).show();
    }


    @Override
    public void onFocusChange (View view, boolean hasFocus) {
        if (hasFocus) {
            switch (view.getId()) {
                case R.id.ap_imageView_profileImage:
                    onClick(view);
                    break;

                default:
                    break;

            }

        }


    }

    @Override
    public void onClick(View view){
        switch (view.getId()){
            case R.id.ap_imageView_profileImage: //If the image profile was clicked
                // check permissions
                if(!hasPermissionsToReadFromGallery())
                { //request permissions
                    ActivityCompat.requestPermissions(this, new String[] {Manifest.permission.READ_EXTERNAL_STORAGE},
                            REQUEST_PERMISSION_STORAGE_READ);
                } else {
                    if (Utitilies.isExternalStorageReadable())
                           pickFromGallery();
                }
                break;

            case R.id.ap_button: //If the button was clicked
                if (mButton.getText().toString().equals(getString(R.string.button_edit))) {
                    setFieldsEditable();
                    mButton.setText(R.string.button_guardar);
                    completeName.requestFocus(View.FOCUS_UP);
                    break;
                }

                //At this point, we know for sure the user wants to CREATE or UPDATE the profile

                if (checkEntryFields()) { // check if all the fields are filled, returns TRUE if everything is OK
                    progressBar.setVisibility(View.VISIBLE);
                    FirebaseUser user = FirebaseAuth.getInstance().getCurrentUser();
                    //Create a new CaminaUser and set all the properties except the ones which belong to Firebase user, which will be added later
                    final CaminaUser caminaUser = new CaminaUser();
                    caminaUser.setCompleteName(completeName.getText().toString());
                    caminaUser.setEmail(email.getText().toString());
                    caminaUser.setAge(Integer.valueOf(age.getText().toString()));
                    caminaUser.setGenre(spinner_GenreSelection.getSelectedItem().toString());
                    caminaUser.setHeight(Integer.valueOf(height.getText().toString()));
                    caminaUser.setWeight(Integer.valueOf(weight.getText().toString()));
                    caminaUser.setCity(city.getText().toString());
                    caminaUser.setCountry(country.getText().toString());
                    if (pictureChanged) {
                        // Save the user picture
                        // Create an instance of FirebaseStorage
                        FirebaseStorage storage = FirebaseStorage.getInstance();
                        // Create a storage reference from our app
                        StorageReference storageRef = storage.getReference();
                        // Create a reference to the name of the picture
                        String fileName = user.getUid() + "Picture.jpg";
                        StorageReference userPictureRef = storageRef.child(fileName);
                        // Create a reference to 'profile_pictures/fileName
                        final StorageReference completePictureRef = storageRef.child("profile_pictures/" + userPictureRef);

                        Bitmap bitmapProfilePic = ((BitmapDrawable) profileImage.getDrawable()).getBitmap();
                        ByteArrayOutputStream baos = new ByteArrayOutputStream();
                        bitmapProfilePic.compress(Bitmap.CompressFormat.JPEG, 90, baos);
                        byte[] data = baos.toByteArray();

                        UploadTask uploadTask = completePictureRef.putBytes(data);

                        Task<Uri> urlTask = uploadTask.continueWithTask(new Continuation<UploadTask.TaskSnapshot, Task<Uri>>() {
                            @Override
                            public Task<Uri> then(@NonNull Task<UploadTask.TaskSnapshot> task) throws Exception {
                                if (!task.isSuccessful()) {
                                    throw task.getException();
                                }

                                // Continue with the task to get the download URL
                                return completePictureRef.getDownloadUrl();
                            }
                        }).addOnCompleteListener(new OnCompleteListener<Uri>() {
                            @Override
                            public void onComplete(@NonNull Task<Uri> task) {
                                if (task.isSuccessful()) {
                                    Uri downloadUri = task.getResult();
                                    //Update the FirebaseAuth user
                                    updateUserProfile(downloadUri, caminaUser);

                                } else {
                                    // Handle failures
                                    // ...
                                }
                            }
                        });


                        uploadTask.addOnFailureListener(new OnFailureListener() {
                            @Override
                            public void onFailure(@NonNull Exception exception) {
                                Toast.makeText(UserProfileActivity.this,
                                        "No se ha podido subir la foto de perfil",
                                        Toast.LENGTH_SHORT).show();
                            }
                        }).addOnSuccessListener(new OnSuccessListener<UploadTask.TaskSnapshot>() {
                            @Override
                            public void onSuccess(UploadTask.TaskSnapshot taskSnapshot) {
                                Toast.makeText(UserProfileActivity.this,
                                        "Foto de perfil subida con éxito",
                                        Toast.LENGTH_SHORT).show();
                            }
                        });
                    }

                    else { //It's not necessary to upload the image profile

                        //Update the FirebaseAuth user and create/update the FireStoreUser
                        updateUserProfile(null, caminaUser);

                    }


                }
            default:
                break;
        }
    }


    void updateUserProfile (Uri profilePictureUri, CaminaUser caminaUser) {

        //Update the FirebaseAuth user
        //First, create a user profile change request with the completeName in the EditText
        UserProfileChangeRequest.Builder profileUpdatesBuilder = new UserProfileChangeRequest.Builder()
                .setDisplayName(completeName.getText().toString());
        // If image was changed, add it to the builder too
        if (pictureChanged && profilePictureUri != null)
            profileUpdatesBuilder.setPhotoUri(profilePictureUri);
        UserProfileChangeRequest profileUpdates = profileUpdatesBuilder.build();
        //Launch the petition
        user.updateProfile(profileUpdates)
                .addOnCompleteListener(task -> {
                    if (task.isSuccessful()) {
                        //If update is successful, create or update existing user in FireStore
                        //Get the pictureUrl from user
                        caminaUser.setPictureUrl(user.getPhotoUrl().toString());
                        //Try to get a reference to an existing FireStore user
                        DocumentReference userRef = firestoreDB.collection("users").document(user.getUid());
                        if (!newProfile)
                        {
                            //It's an existing FireBaseAuth user, with already a FireStore user, so UPDATE profile in FireStore
                            Map<String, Object> updates = new HashMap<>();
                            updates.put("age", caminaUser.getAge());
                            updates.put("city", caminaUser.getCity());
                            updates.put("completeName", caminaUser.getCompleteName());
                            updates.put("country", caminaUser.getCountry());
                            updates.put("genre", caminaUser.getGenre());
                            updates.put("height", caminaUser.getHeight());
                            updates.put("weight", caminaUser.getWeight());
                            if (pictureChanged)
                                updates.put("pictureUrl", caminaUser.getPictureUrl());
                            userRef.update(updates)
                                    .addOnSuccessListener(aVoid -> {
                                        progressBar.setVisibility(View.INVISIBLE);
                                        AlertDialog alertDialog = new AlertDialog.Builder(UserProfileActivity.this).create();
                                        alertDialog.setTitle(R.string.updated_profile_title);
                                        alertDialog.setMessage(getText(R.string.updated_profile_message));
                                        alertDialog.setButton(AlertDialog.BUTTON_NEUTRAL, "OK",
                                                new DialogInterface.OnClickListener() {
                                                    public void onClick(DialogInterface dialog, int which) {
                                                        dialog.dismiss();
                                                        Intent intent = new Intent(UserProfileActivity.this, MapsActivity.class);
                                                        startActivity(intent);
                                                        finish();


                                                    }
                                                });

                                        alertDialog.show();
                                    })
                                    .addOnFailureListener(new OnFailureListener() {
                                        @Override
                                        public void onFailure(@NonNull Exception e) {
                                            progressBar.setVisibility(View.INVISIBLE);
                                            Toast.makeText(UserProfileActivity.this,
                                                    R.string.updated_profile_error,
                                                    Toast.LENGTH_SHORT).show();
                                        }
                                    });

                        }

                        else { //It's a new user in FireStore

                            caminaUser.setUserId(user.getUid());
                            firestoreDB.collection("users").document(user.getUid())//We use the user.getUid as document ID
                                    .set(caminaUser)
                                    .addOnSuccessListener(aVoid -> {
                                        progressBar.setVisibility(View.INVISIBLE);
                                        AlertDialog alertDialog = new AlertDialog.Builder(UserProfileActivity.this).create();
                                        alertDialog.setTitle(R.string.new_profile_title);
                                        alertDialog.setMessage(getText(R.string.new_profile_message));
                                        alertDialog.setButton(AlertDialog.BUTTON_NEUTRAL, "OK",
                                                new DialogInterface.OnClickListener() {
                                                    public void onClick(DialogInterface dialog, int which) {
                                                        newProfile = false;
                                                        dialog.dismiss();
                                                        Intent intent = new Intent(UserProfileActivity.this, MapsActivity.class);
                                                        startActivity(intent);
                                                        finish();
                                                    }
                                                });

                                        alertDialog.show();
                                        pictureChanged = FALSE;
                                    })
                                    .addOnFailureListener(e -> {
                                        progressBar.setVisibility(View.INVISIBLE);
                                        Toast.makeText(UserProfileActivity.this,
                                                R.string.new_profile_error,
                                                Toast.LENGTH_SHORT).show();
                                    });

                        }




                    } else {
                        progressBar.setVisibility(View.INVISIBLE);
                        Toast.makeText(UserProfileActivity.this,
                                "No se ha podido guardar el perfil",
                                Toast.LENGTH_SHORT).show();
                    }
                });
    }







    boolean checkEntryFields() {

        // Check if all the fields have been filled correctly

        if (completeName.getText().toString().equals("")||completeName.getText().toString().length()<3) {
            showMessage(R.string.name_void);
            completeName.requestFocus();
            return (false);
        }

        if (FirebaseAuth.getInstance().getCurrentUser().getPhotoUrl()==null && pictureChanged==false) {
            showMessage (R.string.picture_void);
            return (false);
        }
        if (age.getText().toString().equals("")) {
            showMessage (R.string.age_void);
            age.requestFocus();
            return (false);
        }
        if (height.getText().toString().equals("")) {
            showMessage (R.string.height_void);
            height.requestFocus();
            return (false);
        }
        if (weight.getText().toString().equals("")) {
            showMessage (R.string.weight_void);
            weight.requestFocus();
            return (false);
        }
        if (city.getText().toString().equals("")) {
            showMessage(R.string.city_void);
            city.requestFocus();
            return (false);
        }
        if (country.getText().toString().equals("")) {
            showMessage(R.string.country_void);
            country.requestFocus();
            return (false);

        }
    // If we get to this point, everything is properly filled
    return true;

    }

    @Override
    public void onRequestPermissionsResult(int requestCode, @NonNull String permissions[], @NonNull int[] grantResults) {
        switch (requestCode) {

           case REQUEST_PERMISSION_STORAGE_READ: {
                // If request is cancelled, the result arrays are empty
                if (grantResults.length > 0 && grantResults[0] == PackageManager.PERMISSION_GRANTED) {
                    pickFromGallery();
           }
            }
        }
    }


    private boolean hasPermissionsToReadFromGallery() {
        return ContextCompat.checkSelfPermission(this, Manifest.permission.READ_EXTERNAL_STORAGE)
                == PackageManager.PERMISSION_GRANTED;
    }

    void setFieldsNotEditable () {

        completeName.setFocusable(false);
        completeName.setClickable(false);
        email.setFocusable(false);
        email.setClickable(false);
        spinner_GenreSelection.setEnabled(false);
        profileImage.setFocusable(false);
        profileImage.setClickable(false);
        age.setFocusable(false);
        age.setClickable(false);
        height.setFocusable(false);
        height.setClickable (false);
        weight.setFocusable(false);
        weight.setClickable(false);
        city.setFocusable(false);
        city.setClickable(false);
        country.setFocusable(false);
        country.setClickable(false);

    }

    void setFieldsEditable () {

        completeName.setFocusableInTouchMode(true);
        email.setFocusableInTouchMode(true);
        spinner_GenreSelection.setEnabled(true);
        profileImage.setFocusableInTouchMode(true);
        profileImage.setFocusable(true);
        profileImage.setClickable(true);
        age.setFocusableInTouchMode(true);
        height.setFocusableInTouchMode(true);
        weight.setFocusableInTouchMode(true);
        city.setFocusableInTouchMode(true);
        country.setFocusableInTouchMode(true);

    }

    private void initViews() {
        content = findViewById(R.id.profile_content);
        progressBar = findViewById(R.id.profile_progress);
        completeName = findViewById(R.id.ap_textView_Name_Surnames);
        email = findViewById(R.id.ap_textView_Email);
        profileImage = findViewById(R.id.ap_imageView_profileImage);
        age = findViewById(R.id.ap_editText_Age);
        height = findViewById(R.id.ap_editText_Height);
        weight = findViewById(R.id.ap_editText_Weight);
        city = findViewById(R.id.ap_editText_City);
        country = findViewById(R.id.ap_editText_Country);
        mButton = findViewById(R.id.ap_button);
        spinner_GenreSelection = findViewById(R.id.ap_spinner_Genre);
    }

    private void showLoading() {
        //Show progressbar and hide content
        progressBar.setVisibility(View.VISIBLE);
        content.setVisibility(View.GONE);
    }

    private void showProfileContent() {
        //Show content and hid progressbar
        progressBar.setVisibility(View.GONE);
        content.setVisibility(View.VISIBLE);
    }

}
