package jimenezcarlos.android.uoc.edu.rest.model;

import com.google.gson.annotations.Expose;
import com.google.gson.annotations.SerializedName;

import java.io.Serializable;
import java.util.Date;
import java.util.List;

// This class represents the final Path or Itinerary chosen by the user
// It will be used to store data to Firebase, as well to pass info from activity MetroStopSelection to StartItineraryActivity

public class Path implements Serializable {

    /**
     * No args constructor for use in serialization
     *
     */
    public Path() {
        mSkippedAll = false;
    }

    @SerializedName("originAddress")
    @Expose
    private String mOriginAddress;
    @SerializedName("destinationAddress")
    @Expose
    private String mDestinationAddress;
    @SerializedName("metroLines")
    @Expose
    private List<String> mMetroLines = null;
    @SerializedName("metroLineColors")
    @Expose
    private List<String> mMetroLineColors = null;
    @SerializedName("stationNames")
    @Expose
    private List<StationNamesInLine> mStationNames = null;
    @SerializedName("latFirstMetroStop")
    @Expose
    private String mLatFirstMetroStop;
    @SerializedName("lonFirstMetroStop")
    @Expose
    private String mLonFirstMetroStop;

    @SerializedName("startWalkingDistance")
    @Expose
    private Double mStartWalkingDistance;
    @SerializedName("startWalkingTime")
    @Expose
    private Integer mStartWalkingTime;
    @SerializedName("endWalkingDistance")
    @Expose
    private Double mEndWalkingDistance;
    @SerializedName("endWalkingTime")
    @Expose
    private Integer mEndWalkingTime;
    @SerializedName("skippedStations")
    @Expose
    private Integer mSkippedStations;
    @SerializedName("walkTime")
    @Expose
    private Integer mWalkTime;
    @SerializedName("walkDistance")
    @Expose
    private Double mWalkDistance;
    @SerializedName("totalCalories")
    @Expose
    private Double mCalories;
    @SerializedName("OMSActivity")
    @Expose
    private Double mOMSActivity;
    @SerializedName("Date")
    @Expose
    private Date mDate;
    @SerializedName("skippedAllStationsInFirstMetroLine")
    @Expose
    private Boolean mSkippedAll;



    public Path (String OriginAddress, String DestinationAddress, List<String> MetroLines, List<String> MetroLineColors, List<StationNamesInLine> StationNames, String LatFirstMetroStop, String LonFirstMetroStop, Double StartWalkingDistance, Integer StartWalkingTime, Double EndWalkingDistance,
                 Integer EndWalkingTime, Integer SkippedStations, Double WalkDistance, Integer WalkTime,  Double TotalCalories, Double OMSActivity, Date Date, Boolean SkippedAll) {

        super();

        mOriginAddress = OriginAddress;
        mDestinationAddress = DestinationAddress;

        mMetroLines = MetroLines;
        mMetroLineColors = MetroLineColors;
        mStationNames = StationNames;
        mLatFirstMetroStop = LatFirstMetroStop;
        mLonFirstMetroStop = LonFirstMetroStop;

        mStartWalkingDistance = StartWalkingDistance;
        mStartWalkingTime = StartWalkingTime;
        mEndWalkingDistance = EndWalkingDistance;
        mEndWalkingTime = EndWalkingTime;

        mSkippedStations = SkippedStations;
        mWalkTime = WalkTime;
        mWalkDistance = WalkDistance;
        mCalories = TotalCalories;
        mOMSActivity = OMSActivity;
        mDate = Date;
        mSkippedAll = SkippedAll;


    }

    //Getters
    public String getOriginAddress () { return mOriginAddress;}
    public String getDestinationAddress () {return mDestinationAddress;}
    public List<String> getMetroLines() {
        return mMetroLines;
    }
    public List<String> getMetroLineColors () {return mMetroLineColors;}
    public List<StationNamesInLine> getStationNames() {
        return mStationNames;
    }
    public String getLatFirstMetroStop () { return mLatFirstMetroStop;}
    public String getLonFirstMetroStop () {return mLonFirstMetroStop;}
    public Double getStartWalkingDistance () { return mStartWalkingDistance;}
    public Integer getStartWalkingTime () {return mStartWalkingTime;}
    public Double getEndWalkingDistance () { return mEndWalkingDistance;}
    public Integer getEndWalkingTime () {return mEndWalkingTime;}
    public Integer getSkippedStations () {return mSkippedStations;}
    public Integer getWalkTime () {return mWalkTime;}
    public Double getWalkDistance () {return mWalkDistance;}
    public Double getCalories() {return mCalories;}
    public Double getOMSActivityRatio () {return mOMSActivity;}
    public Date getDate () {return mDate;}
    public Boolean getSkippedAll () {return mSkippedAll;}

    //Setters
    public void setOriginAddress (String OriginAddress) { mOriginAddress = OriginAddress;}
    public void setDestinationAddress (String DestinationAddress) {mDestinationAddress = DestinationAddress;}
    public void setMetroLines(List<String> MetroLines) {
        mMetroLines=MetroLines;
    }
    public void setMetroLineColors(List<String> MetroLineColors) {mMetroLineColors=MetroLineColors;}
    public void setStationNames(List<StationNamesInLine> StationNames) {mStationNames=StationNames;}
    public void setLatFirstMetroStop (String LatFirstMetroStop) {mLatFirstMetroStop = LatFirstMetroStop;}
    public void setLonFirstMetroStop (String LonFirstMetroStop) {mLonFirstMetroStop = LonFirstMetroStop;}
    public void setStartWalkingDistance (Double StartWalkingDistance) {mStartWalkingDistance=StartWalkingDistance;}
    public void setStartWalkingTime (Integer StartWalkingTime) {mStartWalkingTime=StartWalkingTime;}
    public void setEndWalkingDistance (Double EndWalkingDistance) {mEndWalkingDistance=EndWalkingDistance;}
    public void setEndWalkingTime (Integer EndWalkingTime) {mEndWalkingTime = EndWalkingTime;}
    public void setSkippedStations (Integer SkippedStations) {mSkippedStations= SkippedStations;}
    public void setWalkTime (Integer WalkTime) {mWalkTime=WalkTime;}
    public void setWalkDistance (Double WalkDistance) {mWalkDistance=WalkDistance;}
    public void setCalories(Double Calories) {mCalories =Calories;}
    public void setOMSActivityRatio (Double OMSActivityRatio) {mOMSActivity=OMSActivityRatio; }
    public void setDate (Date Date) {mDate=Date;}
    public void setSkippedAll (Boolean SkippedAll) {mSkippedAll=SkippedAll;}

    // Public methods

    // This method is called when user decides to skip all Metro stops in the first Metro Line of its itinerary
    public void deleteFirstMetroLine() {

        mMetroLines.remove(0);
        mMetroLineColors.remove(0);
        mStationNames.remove(0);

    }

}
