
package jimenezcarlos.android.uoc.edu.rest.model.TMBPlannerAPI;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.TimeUnit;

import com.google.gson.annotations.Expose;
import com.google.gson.annotations.SerializedName;

import jimenezcarlos.android.uoc.edu.rest.model.StationNamesInLine;

public class Itinerary implements Serializable {

    @SerializedName("duration")
    @Expose
    private Integer duration;
    @SerializedName("startTime")
    @Expose
    private String startTime;
    @SerializedName("endTime")
    @Expose
    private String endTime;
    @SerializedName("walkTime")
    @Expose
    private Integer walkTime;
    @SerializedName("transitTime")
    @Expose
    private Integer transitTime;
    @SerializedName("waitingTime")
    @Expose
    private Integer waitingTime;
    @SerializedName("walkDistance")
    @Expose
    private Double walkDistance;
    @SerializedName("walkLimitExceeded")
    @Expose
    private Boolean walkLimitExceeded;
    @SerializedName("elevationLost")
    @Expose
    private Integer elevationLost;
    @SerializedName("elevationGained")
    @Expose
    private Integer elevationGained;
    @SerializedName("transfers")
    @Expose
    private Integer transfers;
    @SerializedName("legs")
    @Expose
    private List<Leg> legs = null;
    @SerializedName("tooSloped")
    @Expose
    private Boolean tooSloped;

    /**
     * No args constructor for use in serialization
     */
    public Itinerary() {
    }

    /**
     * @param walkDistance
     * @param transitTime
     * @param walkTime
     * @param waitingTime
     * @param walkLimitExceeded
     * @param elevationGained
     * @param tooSloped
     * @param duration
     * @param transfers
     * @param legs
     * @param elevationLost
     * @param startTime
     * @param endTime
     */
    public Itinerary(Integer duration, String startTime, String endTime, Integer walkTime, Integer transitTime, Integer waitingTime, Double walkDistance, Boolean walkLimitExceeded, Integer elevationLost, Integer elevationGained, Integer transfers, List<Leg> legs, Boolean tooSloped) {
        super();
        this.duration = duration;
        this.startTime = startTime;
        this.endTime = endTime;
        this.walkTime = walkTime;
        this.transitTime = transitTime;
        this.waitingTime = waitingTime;
        this.walkDistance = walkDistance;
        this.walkLimitExceeded = walkLimitExceeded;
        this.elevationLost = elevationLost;
        this.elevationGained = elevationGained;
        this.transfers = transfers;
        this.legs = legs;
        this.tooSloped = tooSloped;
    }

    public Integer getDuration() {
        return duration;
    }

    public String getDurationFormatted() {

        String durationFormatted = "";
        long minutes = TimeUnit.SECONDS.toMinutes(this.duration);

        if (minutes < 100) {

            durationFormatted = minutes + " min";

        } else {

            long hours = TimeUnit.SECONDS.toHours(this.duration);
            minutes = TimeUnit.SECONDS.toMinutes(this.duration) - (TimeUnit.SECONDS.toHours(this.duration) * 60);

            if (hours > 0) durationFormatted = durationFormatted + " " + hours + " h";
            if (minutes > 0) durationFormatted = durationFormatted + " " + minutes + "min";
        }
        return durationFormatted;

    }

    public void setDuration(Integer duration) {
        this.duration = duration;
    }

    public String getStartTime() {
        return startTime;
    }

    public void setStartTime(String startTime) {
        this.startTime = startTime;
    }

    public String getEndTime() {
        return endTime;
    }

    public void setEndTime(String endTime) {
        this.endTime = endTime;
    }

    public Integer getWalkTime() {
        return walkTime;
    }

    public String getWalkTimeFormatted() {

        String walkTimeFormatted = "";
        long minutes = TimeUnit.SECONDS.toMinutes(this.walkTime);

        if (minutes == 0) {
            walkTimeFormatted = this.walkTime + " seg";

        } else if (minutes < 60) {

            walkTimeFormatted = minutes + " min";
            long remainingSecs = this.walkTime - minutes * 60;
            if (minutes < 6 && remainingSecs > 0) {

                walkTimeFormatted = walkTimeFormatted + " y " + remainingSecs + " seg";
            }

        } else {


            long hours = TimeUnit.SECONDS.toHours(this.walkTime);
            minutes = TimeUnit.SECONDS.toMinutes(this.walkTime) - (TimeUnit.SECONDS.toHours(this.walkTime) * 60);

            if (hours > 0) walkTimeFormatted = walkTimeFormatted + " " + hours + " h";
            if (minutes > 0) walkTimeFormatted = walkTimeFormatted + ", " + minutes + "min";

        }

        return walkTimeFormatted;
    }


    public void setWalkTime(Integer walkTime) {
        this.walkTime = walkTime;
    }

    public Integer getTransitTime() {
        return transitTime;
    }

    public void setTransitTime(Integer transitTime) {
        this.transitTime = transitTime;
    }

    public Integer getWaitingTime() {
        return waitingTime;
    }

    public void setWaitingTime(Integer waitingTime) {
        this.waitingTime = waitingTime;
    }

    public Double getWalkDistance() {
        return walkDistance;
    }

    public String getWalkDistanceFormatted() {

        String walkDistanceFormatted;
        Double auxWalkDistance;
        if (this.walkDistance >= 1000) {
            auxWalkDistance = (double) Math.round(this.walkDistance / 1000 * 100d) / 100d;
            walkDistanceFormatted = String.format("%.2f", auxWalkDistance) + " km"; //Round to 2 decimal
        } else {
            auxWalkDistance = this.walkDistance;
            walkDistanceFormatted = String.format("%.0f", auxWalkDistance) + " m"; //No decimals

        }

        return walkDistanceFormatted;

    }

    public void setWalkDistance(Double walkDistance) {
        this.walkDistance = walkDistance;
    }

    public Boolean getWalkLimitExceeded() {
        return walkLimitExceeded;
    }

    public void setWalkLimitExceeded(Boolean walkLimitExceeded) {
        this.walkLimitExceeded = walkLimitExceeded;
    }

    public Integer getElevationLost() {
        return elevationLost;
    }

    public void setElevationLost(Integer elevationLost) {
        this.elevationLost = elevationLost;
    }

    public Integer getElevationGained() {
        return elevationGained;
    }

    public void setElevationGained(Integer elevationGained) {
        this.elevationGained = elevationGained;
    }

    public Integer getTransfers() {
        return transfers;
    }

    public void setTransfers(Integer transfers) {
        this.transfers = transfers;
    }

    public List<Leg> getLegs() {
        return legs;
    }

    public void setLegs(List<Leg> legs) {
        this.legs = legs;
    }

    public Boolean getTooSloped() {
        return tooSloped;
    }

    public void setTooSloped(Boolean tooSloped) {
        this.tooSloped = tooSloped;
    }

    // This method returns the first Metro (SUBWAY) leg in this Itinerary
    public int getFirstMetroLeg() {

        int i;
        boolean isThereAMetroLeg = false;
        for (i = 0; i < this.legs.size(); i++) {
            if (this.legs.get(i).getMode().equals("SUBWAY")){
                isThereAMetroLeg = true;
                break;
            }
        }
        if (isThereAMetroLeg) return (i);
            else return -1;
    }
    // This method returns the second Metro (SUBWAY) leg in this Itinerary
    // This is useful in cases where the user decides to skip the first metro line
    public int getSecondMetroLeg() {

        int i;
        int metroLegsFound=0;
        for (i = 0; i < this.legs.size(); i++) {
            if (this.legs.get(i).getMode().equals("SUBWAY")){
                metroLegsFound++;
                if (metroLegsFound==2) break;
            }
        }
        if (metroLegsFound==2) return (i);
        else return -1;
    }
    // Returns the total number of Metro Lines included in the itinerary
    public int getTotalMetroLegs() {

        int i;
        int metroLegsFound=0;
        for (i = 0; i < this.legs.size(); i++) {
            if (this.legs.get(i).getMode().equals("SUBWAY")){
                metroLegsFound++;

            }
        }
        return (metroLegsFound);
    }

    public Double getWalkingDistanceToFirstMetroStation() {


        int i;
        Double distanceToFirstMetroStation = 0.0;

        for (i = 0; i < this.legs.size(); i++) {
            if (this.legs.get(i).getMode().equals("SUBWAY"))
                break;
            else if (this.legs.get(i).getMode().equals("WALK")) {
                distanceToFirstMetroStation = distanceToFirstMetroStation + this.legs.get(i).getDistance();
            }
        }

        return distanceToFirstMetroStation;

    }

    public int getWalkingTimeToFirstMetroStation() {

        int i;
        int walkingTimeToFirstMetroStation = 0;
        for (i = 0; i < this.legs.size(); i++) {
            if (this.legs.get(i).getMode().equals("SUBWAY"))
                break;
            else if (this.legs.get(i).getMode().equals("WALK")) {
                walkingTimeToFirstMetroStation = walkingTimeToFirstMetroStation + this.legs.get(i).getDuration();
            }
        }

        return walkingTimeToFirstMetroStation;

    }


    public Double getWalkingDistanceAddingAllLegs() {

        int i;
        Double totalWalkingDistance = 0.0;

        for (i = 0; i < this.legs.size(); i++) {
            if (this.legs.get(i).getMode().equals("WALK")) {
                totalWalkingDistance = totalWalkingDistance + this.legs.get(i).getDistance();
            }
        }
        return totalWalkingDistance;
    }

    public int getWalkingTimeAddingAllLegs() {

        int i;
        int totalWalkingTime = 0;
        for (i = 0; i < this.legs.size(); i++) {
            if (this.legs.get(i).getMode().equals("WALK")) {
                totalWalkingTime = totalWalkingTime + this.legs.get(i).getDuration();
            }
        }

        return totalWalkingTime;

    }
    // This method returns the names of the different Metro lines included in this itinerary
    public List<String> getMetroLines() {

        int i;
        List<String> metroLines = new ArrayList<>();

        for (i = 0; i < this.legs.size(); i++) {
            if (this.legs.get(i).getMode().equals("SUBWAY")) {
                metroLines.add(this.legs.get(i).getRoute());
            }
        }
        return metroLines;
    }
    // This method returns the colors of the different Metro lines included in this itinerary
    public List<String> getMetroLineColors () {

        int i;
        List<String> metroLineColors = new ArrayList<>();

        for (i = 0; i < this.legs.size(); i++) {
            if (this.legs.get(i).getMode().equals("SUBWAY")) {
                metroLineColors.add(this.legs.get(i).getRouteColor());
            }
        }
        return metroLineColors;
    }

    // This method returns the names of all the Metro stations, of all Metro Lines, included in this itinerary
    // The returned value is a list of objects StationNamesInLine
    // Each object StationNamesInLine is a list of the names of all the stations included in an specific Metro line included in this itinerary
    public List<StationNamesInLine> getStationNames() {

        List<StationNamesInLine> stationNames = new ArrayList<>();

        for (int i = 0; i < this.legs.size(); i++) { //For every Leg
            if (this.legs.get(i).getMode().equals("SUBWAY")) { //If it's a Metro Leg

                List<String> stationNamesInThisLeg = new ArrayList<>();
                stationNamesInThisLeg.add (this.legs.get(i).getFrom().getName());
                for (TransitStop station: this.legs.get(i).getIntermediateStops())
                {
                    stationNamesInThisLeg.add (station.getName());
                }
                stationNamesInThisLeg.add(this.legs.get(i).getTo().getName());
                StationNamesInLine aux = new StationNamesInLine(stationNamesInThisLeg);
                stationNames.add (aux);
            }
        }

        return stationNames;
    }


    public Double getWalkingDistanceFromLastMetroStationToDestination() {


        int i;
        Double distanceFromLastMetroStationToDestination = 0.0;

        for (i = this.legs.size()-1; i >= 0; i--) {
            if (this.legs.get(i).getMode().equals("SUBWAY"))
                break;
            else if (this.legs.get(i).getMode().equals("WALK")) {
                distanceFromLastMetroStationToDestination = distanceFromLastMetroStationToDestination + this.legs.get(i).getDistance();
            }
        }

        return distanceFromLastMetroStationToDestination;

    }

    public Integer getWalkingTimeFromLastMetroStationToDestination() {

        int i;
        int walkingTomeFromLastMetroStationToDestination = 0;
        for (i = this.legs.size()-1; i >= 0; i--) {
            if (this.legs.get(i).getMode().equals("SUBWAY"))
                break;
            else if (this.legs.get(i).getMode().equals("WALK")) {
                walkingTomeFromLastMetroStationToDestination = walkingTomeFromLastMetroStationToDestination + this.legs.get(i).getDuration();
            }
        }

        return walkingTomeFromLastMetroStationToDestination;

    }

    //This method returns duration in seconds (walking or by Metro) from origin to given Metro stop of the first Metro leg

    public Integer getItineraryDurationFromOriginToChosenMetroStop (int metroStopNumber) {

        int durationFromOriginToChosenMetroStop = 0;

        int i=0;

        // First we add the duration of all the walking legs from origin to first Metro leg
        while (this.legs.get(i).getMode().equals("WALK"))

        {
            durationFromOriginToChosenMetroStop = durationFromOriginToChosenMetroStop + this.legs.get(i).getDuration();
            i++;
        }

        // At this point, i keeps the value of the first Metro leg
        // Metro Legs have following data structure:
            // TransitStop from --> Metro stop of departure
            // TransitStop to --> Metro stop of destination)
            // List<TransitStop> --> intermediateStops
        // And a TransitStop has this two properties we will be using to calculate duration between metro stops (by Metro)
            //Integer arrival --> Arrival time (in Epoch timestamp format: milliseconds since 00:00:00 GMT on January 1, 1970)
            //Integer departure --> Departure time (in Epoch timestamp format)

        // So to calculate the time (in seconds) to get by Metro from the initial Metro stop of this leg
        // to the given metro stop, we will have to deduct the departure time of origin station to
        // the arrival time of the given station. And so the total duration will be:

        durationFromOriginToChosenMetroStop = durationFromOriginToChosenMetroStop
                                            + (int) ((this.legs.get(i).getIntermediateStops().get(metroStopNumber-1).getArrival() -  this.legs.get(i).getFrom().getDeparture())/1000);

        return durationFromOriginToChosenMetroStop;

    }

    //This method returns duration in seconds (walking of by Metro) from beginning of given leg to the end of itinerary

    public Integer getItineraryDurationFromGivenLegToEnd (int numLeg) {

        int durationFromGivenLegtoEnd = 0;

        for (int i = numLeg; i < this.legs.size(); i++)

            durationFromGivenLegtoEnd = durationFromGivenLegtoEnd + this.legs.get(i).getDuration();

        durationFromGivenLegtoEnd = durationFromGivenLegtoEnd + this.waitingTime/this.getTotalMetroLegs(); // We add the proportional part of the waiting time, to be consistent
        return durationFromGivenLegtoEnd;


    }

    //This method returns duration in seconds (walking of by Metro) from the start of itinerary to the beginning of given leg
    public Integer getItineraryDurationFromStartToGivenLeg (int numLeg) {

        int durationFromStartToGivenLeg = this.duration;

        for (int i = 0; i < numLeg; i++)

            durationFromStartToGivenLeg = durationFromStartToGivenLeg - this.legs.get(i).getDuration();

        return durationFromStartToGivenLeg;

    }


}




