package jimenezcarlos.android.uoc.edu.rest.service;

import com.google.gson.JsonObject;

import java.util.Map;

import jimenezcarlos.android.uoc.edu.rest.constants.ApiTMBConstants;

import jimenezcarlos.android.uoc.edu.rest.model.TMBTransitAPI.TMBTransitMetroLinesAPIResponse;
import jimenezcarlos.android.uoc.edu.rest.model.TMBTransitAPI.TMBTransitStationsInLineAPIResponse;
import retrofit2.Call;
import retrofit2.http.GET;
import retrofit2.http.Path;

import retrofit2.http.QueryMap;

public interface TMBService {

    // Every method of an interface represents one possible API call.
    // It must have a HTTP annotation (GET, POST, etc.) to specify the request type and the relative URL.
    // The return value wraps the response in a Call object with the type of the expected result.

    // I use replacement blocks and query parameters to adjust the URL.
    // A replacement block is added to the relative URL with {}.
    // With the help of the @Path annotation on the method parameter,
    // the value of that parameter is bound to the specific replacement block.

    @GET(ApiTMBConstants.TMB_ALL_METRO_STATIONS_ENDPOINT)
    Call<TMBTransitMetroLinesAPIResponse> getAllMetroStations();

    @GET(ApiTMBConstants.TMB_METRO_STATIONS_BY_LINE_ENDPOINT)
    Call<TMBTransitStationsInLineAPIResponse> getMetroLineStations(@Path("line_code") String mTMBLine_code);


    // Since there are multiple query parameters, we use a Map

    @GET(ApiTMBConstants.TMB_PLANNER_ENDPOINT)
    Call<JsonObject> getPlannedRoute(@QueryMap Map<String, String> options);

}
