package jimenezcarlos.android.uoc.edu.util;

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.Canvas;
import android.graphics.drawable.Drawable;
import android.os.Environment;
import android.util.Log;

import androidx.annotation.NonNull;
import androidx.core.content.ContextCompat;

import com.google.android.gms.maps.GoogleMap;
import com.google.android.gms.maps.model.BitmapDescriptor;
import com.google.android.gms.maps.model.BitmapDescriptorFactory;
import com.google.android.gms.maps.model.LatLng;
import com.google.android.gms.maps.model.MarkerOptions;


import java.io.IOException;
import java.util.List;
import java.util.Locale;
import java.util.concurrent.TimeUnit;


import jimenezcarlos.android.uoc.edu.rest.adapter.TMBAdapter;

import jimenezcarlos.android.uoc.edu.rest.model.TMBTransitAPI.StationFeature;
import jimenezcarlos.android.uoc.edu.rest.model.TMBTransitAPI.TMBTransitStationsInLineAPIResponse;
import retrofit2.Call;
import retrofit2.Callback;
import retrofit2.Response;



public class Utitilies {

    private Call<TMBTransitStationsInLineAPIResponse> callTMBTransitStationsInLineAPIResponse;

    private GoogleMap mMap;
    private Context mContext; //In case we need a Context reference to access resources

    // Line codes of Metro Lines according to TMB API
    private String[] METROLINECODES = {"1", "4", "2", "11", "94", "91", "101", "99", "3", "5", "104"};


    public Utitilies(GoogleMap map, Context context) {
        mMap = map;
        mContext = context;

    }



    public void AddMetroStops() {

        //TODO: showLoading();

        for (String metroLineCode:METROLINECODES){

            callTMBTransitStationsInLineAPIResponse = new TMBAdapter().getMetroLineStations(String.valueOf(metroLineCode));
            callTMBTransitStationsInLineAPIResponse.enqueue(new Callback<TMBTransitStationsInLineAPIResponse>() {
                @Override
                public void onResponse(@NonNull Call<TMBTransitStationsInLineAPIResponse> call, @NonNull Response<TMBTransitStationsInLineAPIResponse> response) {
                    TMBTransitStationsInLineAPIResponse responseTMBTransitAPI = response.body();
                    if (responseTMBTransitAPI != null) {

                        drawMetroStops (responseTMBTransitAPI.getFeatures());



                    } else {

                        Log.d("AddMetroStops", "Recibido error del servidor");
                        //TODO: handle answer
                    }
                }

                @Override
                public void onFailure(@NonNull Call<TMBTransitStationsInLineAPIResponse> call, @NonNull Throwable t) {


                    if (t instanceof IOException) {
                        Log.d("AddMetroStops", "Network failure :( inform the user and possibly retry");
                    }
                    else {

                        Log.d("AddMetroStops", "Conversion issue");


                    }
                }
            });

        }


    }


    public void drawMetroStops (List<StationFeature> metroStops)
    {

        // Auxiliary class to get the name of drawable resources of transit icon lines
        LineToTransitIconDrawable iconLines;
        //Instantiate a new object of class LineToTransitIconDrawable
        iconLines = new LineToTransitIconDrawable ();

        //For each metroStop
        for (StationFeature mS:metroStops) {


            String resource = iconLines.getResourceName(mS.getProperties().getNOMLINIA());
            int id = mContext.getResources().getIdentifier(resource, "drawable", mContext.getPackageName());

            mMap.addMarker(new MarkerOptions()
                .position(new LatLng(mS.getGeometry().getCoordinates().get(1), mS.getGeometry().getCoordinates().get(0)))
                .title(mS.getProperties().getNOMESTACIO())
                .snippet(mS.getProperties().getDESCSERVEI())
                .icon(bitmapDescriptorFromVector(mContext, id)));

        }

    }

    private BitmapDescriptor bitmapDescriptorFromVector(Context context, int vectorResId) {
        Drawable vectorDrawable = ContextCompat.getDrawable(context, vectorResId);
        vectorDrawable.setBounds(0, 0, (int)(vectorDrawable.getIntrinsicWidth()*0.15), (int)(vectorDrawable.getIntrinsicHeight()*0.15));
        Bitmap bitmap = Bitmap.createBitmap((int)(vectorDrawable.getIntrinsicWidth()*0.15), (int)(vectorDrawable.getIntrinsicHeight()*0.15), Bitmap.Config.ARGB_8888);
        Canvas canvas = new Canvas(bitmap);
        vectorDrawable.draw(canvas);
        return BitmapDescriptorFactory.fromBitmap(bitmap);
    }

    public static String FormatDistanceDoubleToString (Double distance) {

        String distanceFormated;
        Double auxWalkDistance;

        if (distance>=1000 && distance <100000)
        {
            auxWalkDistance = (double)Math.round(distance/1000 * 100d) / 100d;
            distanceFormated = String.format(Locale.getDefault(),"%(,.2f", auxWalkDistance)+" km"; //Round to 2 decimal and miles separator
        } else if (distance>=10000)
        {

            auxWalkDistance = (double)Math.round(distance/1000 * 100d) / 100d;
            distanceFormated = String.format(Locale.getDefault(),"%(,.0f", auxWalkDistance)+" km"; //No decimals and miles separator

        }

        else {
            auxWalkDistance = distance;
            distanceFormated = String.format(Locale.getDefault(), "%(,.0f", auxWalkDistance)+" m"; //No decimals and miles separator

        }
        return distanceFormated;
    }


    // Receives time in seconds and format to a string indicating hours and minutes
    public static String FormatTimeIntegerToString (int time) {
        String walkTimeFormatted="";
        long minutes = TimeUnit.SECONDS.toMinutes(time);

        if (minutes==0)

        {
            walkTimeFormatted = time + " seg";

        }


        else if (minutes<60) {

            long remainingSeconds = time - minutes*60;
            if (remainingSeconds>=30) minutes++; // Round to the nearest integer

            walkTimeFormatted = minutes + " min";

        }
        else {


            long hours = TimeUnit.SECONDS.toHours(time);
            minutes = TimeUnit.SECONDS.toMinutes(time) - (TimeUnit.SECONDS.toHours(time) * 60);

            if (hours > 0) walkTimeFormatted = hours + " h";
            if (minutes > 0) walkTimeFormatted = walkTimeFormatted + " " + minutes + " min";

        }

        return walkTimeFormatted;
    }



    /**
     * Checks if external storage is available to read
     */

    public static boolean isExternalStorageReadable() {
        String state = Environment.getExternalStorageState();
        if (Environment.MEDIA_MOUNTED.equals(state) || Environment.MEDIA_MOUNTED_READ_ONLY.equals(state)) {
            return true;
        }
        return false;
    }





}
